/*
 * $Id: OverviewObject.java,v 1.18 2002/07/07 22:08:24 js Exp $
 *
 */

/**
 * Objektklasse.
 * speichert eine allgemeine Uebersicht aller empfangenen und bearbeiteten,
 * sowie eventuell interpolierter Daten
 * @author Patrick Gaertner (pg)
 *
 */

public class OverviewObject 
{
	
	// Attribute
	
	/** eigene Position in absoluten Koordinaten */
	private XyObject selfPos = new XyObject();
	/** Ballposition in absoluten Koordinaten */
	private XyzObject ballPos = null;
	/** Teamspielerpositionen in absoluten Koordinaten */
	private XyzObject[] playerPos = new XyzObject[11];
	/** Feindspielerpositionen in absoluten Koordinaten */
	private XyzObject[] enemyPos = new XyzObject[11];
	/** Spielerpositionen in absoluten Koordinaten fuer unbekannte Mitspieler */
	private XyzObject[] unknownPos;
	/** gehoerte Nachricht */
	private String[] hear = null;
	/** Ausdauer des Spielers */
	private double stamina = 4444;
	/** Kopfdrehung */
	private double headangle = 0;
	/** Koerperdrehung */
	private double bodyangle = 4444;
    /** Spielernummer des angebruellten Spielers, "12" fuer niemanden */
    private int me = 12;
    /** Zielkoordinaten fuer diesen Spieler */
    private XyObject target = new XyObject();
    private String spielmodus;
	private boolean seeBall = false;
    private double ballWinkel = 0;
    private int timestep = 0;
	private double speedAmount;
	private int speedDir;
	// Konstruktoren
	
	/** Standartkonstruktor ohne Parameter
	 * @author pg
	 */
	public OverviewObject()
	{
	}
	
	/** Konstruktor mit allen Werten als Parametern
	 * @author pg
	 * @param selfPosIn Position des Spielers 
	 * @param ballPosIn Position des Balls 
	 * @param playerPosIn Positionen der Teamspieler
	 * @param enemyPosIn Positionen der Feindspieler
	 * @param unknownPosIn Positionen nicht exakt zuzuordnender Spieler
	 * @param hearIn zuletzt gehoerte Nachricht
	 * @param staminaIn aktuelle Ausdauer
	 * @param headangleIn Winkel der Kopfdrehung
	 * @param bodyangleIn Winkel der Koerperdrehung
	 */
	public OverviewObject(XyObject selfPosIn, XyzObject ballPosIn, XyzObject[] playerPosIn, XyzObject[] enemyPosIn, XyzObject[] unknownPosIn, String[] hearIn, double staminaIn, double headangleIn, double bodyangleIn)
	{
		this.selfPos = selfPosIn;
		this.ballPos = ballPosIn;
		this.playerPos = playerPosIn;
		this.enemyPos = enemyPosIn;
		this.unknownPos = unknownPosIn;
		this.hear = hearIn;
		this.stamina = staminaIn;
		this.headangle = headangleIn;
		this.bodyangle = bodyangleIn;
	}
	
	// get & set Methoden
	
	/** gibt die eigene Position zurueck
	 * @author pg
	 * @return selfPos XyObject mit der Position
	 */
	public XyObject getSelfPos()
	{
		return this.selfPos;
	}
	
	/** gibt die Ballposition zurueck
	 * @author pg
	 * @return ballPos XyzObject mit der Ballposition
	 */
	public XyzObject getBallPos()
	{
		return this.ballPos;
	}
    
    public double getBallWinkel()
    {
        return this.ballWinkel;
    }
	
	/** gibt seeBall zurueck
	 * @author pg
	 * @return seeBall seh ich den Ball?
	 */
	public boolean getSeeBall()
	{
		return this.seeBall;
	}
	
	/** gibt die Positionen der Teamspieler zurueck
	 * @author pg
	 * @return playerPos Array mit XyzObjecten mit den Positionen
	 */
	public XyzObject[] getPlayerPos()
	{
		return this.playerPos;
	}
	
	/** gibt die Position eines Teamspielers zurueck
	 * @author pg
	 * @param no Spielernummer des Spielers
	 * @return playerPos XyzObject mit der Position
	 */
	public XyzObject get1PlayerPos(int no)
	{
	  if (no < this.playerPos.length)
  		return this.playerPos[(no)];
	  else
	    return null;
	}
	
	/** gibt die Positionen der Feindspieler zurueck
	 * @author pg
	 * @return enemyPos Array mit XyzObjecten mit den Positionen
	 */
	public XyzObject[] getEnemyPos()
	{
		return this.enemyPos;
	}
	
	/** gibt die Position eines Feindspielers zurueck
	 * @author pg
	 * @param no Spielernummer des Spielers
	 * @return enemyPos XyzObject mit der Position
	 */
	public XyzObject get1EnemyPos(int no)
	{
	  if (no < this.enemyPos.length)
	    return this.enemyPos[(no)];
	  else
	    return null;
	}
	
	/** gibt die Positionen der unbekannten Spieler zurueck
	 * @author pg
	 * @return unknownPos Array mit XyzObjecten mit den Positionen
	 */
	public XyzObject[] getUnknownPos()
	{
		return this.unknownPos;
	}
	
	/** gibt den hear-String zurueck
	 * @author pg
	 * @return hear String mit der zuletzt gehoerten Nachricht
	 */
	public String[] getHear()
	{
		return this.hear;
	}
	
	/** gibt die Ausdauer zurueck
	 * @author pg
	 * @return stamina Ausdauer des Spielers
	 */
	public double getStamina()
	{
		return this.stamina;
	}
    
    /**gibt den aktuellen Timestep zurck
     * @author ms
     * @return timestep der momentane Timestep
     * 
     */
    public int getTimestep()
    {
        return this.timestep;
    }
	
	/** gibt die headangle zurueck
	 * @author pg
	 * @return headangle winkel der Kopfdrehung
	 */
	public double getHeadangle()
	{
		return this.headangle;
	}
	
	/** gibt die bodyangle zurueck
	 * @author pg
	 * @return bodyangle winkel der Koerperdrehung
	 */
	public double getBodyangle()
	{
		return this.bodyangle;
	}

	/** gibt die Nummer des Spielers zurueck
	 * @author pg
	 * @return me Spielernummer des gemeinten Spielers
	 */
	public int  getMe()
	{
		return this.me;
	}

	/** gibt die Zielkoordinaten zurueck
	 * @author pg
	 * @return ballPos XyObject mit den Koordinaten auf die sich der gemeinte Spieler bewegen soll
	 */
	public XyObject getTarget()
	{
		return this.target;
	}
    
    /** gibt den aktuellen Spielmodus zurck
     * @author Marcus Schumacher (ms)
     * @return spielmodus String mit dem Spielstatus (before_kick_off, free_kick,...)
     */
    public String getSpielmodus()
    {
        return this.spielmodus;
    }
	
	public int getSpeedDir()
	{
	  return speedDir;
	}
	
	public double getSpeedAmount()
	{
	  return speedAmount;
	}
	
	
	/** setzt die eigene Position
	 * @author pg
	 * @param koord Koordinaten als XyObject denen selfPos zugewiesen wird
	 */
	public void setSelfPos(XyObject koord)
	{
		this.selfPos = koord;
	}
	
	/** setzt die ballposition
	 * @author pg
	 * @param koord Koordinaten als XyzObject denen ballPos zugewiesen wird
	 */
	public void setBallPos(XyzObject koord)
	{
		this.ballPos = koord;
	}
	
	/** setzt den Array mit den Teamspielerpositionen
	 * @author pg
	 * @param koord Array mit XyzObjecten mit den Koordinaten der Spieler
	 */
	public void setPlayerPos(XyzObject[] koord)
	{
		this.playerPos = koord;
	}
	
	/** setzt den Array mit den Feindspielerpositionen
	 * @author pg
	 * @param koord Array mit XyzObjecten mit den Koordinaten der Spieler
	 */
	public void setEnemyPos(XyzObject[] koord)
	{
		this.enemyPos = koord;
	}
	
	/** setzt den Array mit den Positionen der unbekannten Spieler
	 * @author pg
	 * @param koord Array mit XyzObjecten mit den Koordinaten der Spieler
	 */
	public void setUnknownPos(int pos, XyzObject koord)
	{
		this.unknownPos[(pos)] = koord;
	}

	public void setUnknownPos(XyzObject[] koord)
	{
		this.unknownPos = koord;
	}
	
	/** setzt den hearstring
	 * @author pg
	 * @param hearIn text der hear zugewiesen wird
	 */
	public void  setHear(String[] hearIn)
	{
		this.hear = hearIn;
	}
	
	/** setzt die Ausdauer
	 * @author pg
	 * @param staminaIn Wert der stamina zugewiesen wird
	 */
	public void setStamina(double staminaIn)
	{
		this.stamina = staminaIn;
	}
	
	/** setzt die kopfdrehung
	 * @author pg
	 * @param headangleIn Winkel der headangle zugewiesen wird
	 */
	public void setHeadangle(double headangleIn)
	{
		this.headangle = headangleIn;
	}
    
    public void setBallWinkel(double ballWnklIn)
    {
        this.ballWinkel = ballWnklIn;
    }
	
	/** setzt die koerperdrehung
	 * @author pg
	 * @param bodyangleIn Winkel der bodyangle zugewiesen wird
	 */
	public void setBodyangle(double bodyangleIn)
	{
		this.bodyangle = bodyangleIn;
	}

	/** setzt die Spielernummer
	 * @author pg
	 * @param meIn Nummer die me zugewiesen wird
	 */
	public void setMe(int meIn)
	{
		this.me = meIn;
	}

	/** setzt die Zielposition des Spielers
	 * @author pg
	 * @param targetIn Koordinaten die target zugewiesen werden
	 */
	public void setTarget(XyObject targetIn)
	{
		this.target = targetIn;
	}
    
    
    /** setzt den aktuellen Spielmodus
     * @author ms
     * @param modus  der momentane Spielmodus
     */
    public void setSpielmodus(String modus)
    {
        this.spielmodus = modus;
    }
	
    public void setTimestep (int timest)
    {
        this.timestep = timest;
    }
    
	/** setzt seeBall auf true / false
	 * @author pg
	 * @param seeBallIn Wert auf den seeBallIn
	 */
	public void setSeeBall(boolean seeBallIn)
	{
		this.seeBall = seeBallIn;
	}
	
	public void setSpeedAmount(double amount)
	{
	  speedAmount= amount;
	}
	
	public void setSpeedDir(int dir)
	{
	  speedDir= dir;
	}
	
}

/*
 * $Log: OverviewObject.java,v $
 * Revision 1.18  2002/07/07 22:08:24  js
 * SpeedAmount und SpeeDir eingefgt
 *
 * Revision 1.17  2002/07/06 23:08:36  ms
 * - get&set Methoden fr timestep
 *
 * Revision 1.16  2002/07/06 16:16:11  fg
 * Zustzliche setUnknownPos
 *
 * Revision 1.15  2002/07/06 02:09:31  js
 * Version 1.12 wieder hergestellt :-)
 *
 * Revision 1.14  2002/07/06 02:05:18  js
 * weitere kleine Fehlerbehebung
 *
 * Revision 1.13  2002/07/06 02:01:18  js
 * get1PlayerPos und get1EnemyPos korrigiert (ArrayIndex)
 *
 * Revision 1.12  2002/07/06 00:05:59  ms
 * - BallWinkel
 *
 * Revision 1.11  2002/07/05 22:14:00  pg
 * seeBall boolean eingefuegt
 *
 * Revision 1.10  2002/07/01 19:34:42  ms
 * *** empty log message ***
 *
 * Revision 1.9  2002/07/01 18:37:26  ms
 * -uups noch nen Fehler entdeckt
 *
 * Revision 1.8  2002/07/01 15:39:59  ms
 * - setSpielmodus hinzugefgt
 *
 * Revision 1.7  2002/06/26 09:26:45  pg
 * stamina jetzt in double
 *
 * Revision 1.6  2002/06/25 21:01:27  fg
 * headAngle standardmssig auf 0 gesetzt (da ist der auch von anfang an) ;)
 *
 * Revision 1.5  2002/06/25 20:34:35  fg
 * Wenn versucht wird auf einen Spieler (mit get1* ) zuzugreifen und das Array mal wieder nicht existiert wird jetzt ein null-pointer zurckgegeben anstatt abzustrzen
 *
 * Revision 1.4  2002/06/20 11:05:10  pg
 * - Attribute fuer Hear/Say eingefuegt + get/set-methoden
 *
 * Revision 1.3  2002/06/17 10:40:55  pg
 * - bodyangle / headangle hinzugefuegt + get/set. initalwert: 4444
 * - stamina 4444
 * - JavaDoc
 *
 * Revision 1.2  2002/06/11 18:31:40  ms
 * *** empty log message ***
 *
 * Revision 1.1.1.1  2002/06/05 10:38:57  pg
 * intital version
 *
 *
 */

