/*
 * $Id: Player.java,v 1.85 2002/07/14 12:07:31 sg Exp $
 *
 */

/**
 * SpielerKlasse.
 * stellt den spielern Fussballfertigkeiten zur verfuegung.
 * @author Florian Grassl (fg)
 * @author Patrick Gaertner (pg)
 *
 */

import strategie.*;
import comPack.*;
import java.lang.Math;

public class Player
	extends java.lang.Thread

{
	// Referenzen

	/** Referenz auf Weltbild */
	protected Weltbild weltb;
	/** Referenz auf Taktik */
	protected Taktik takt;
	/** Referenz auf Parser */
	protected Parser pars;

	// Attribute

	/** Spielernummer 1 bis 11 */
	private int spielernummer;
	/** Spielerposition, z.B. "t" fuer "Torwart" oder "a" fuer "Abwehrspieler" etc */
	private char position;
	/** aktuelle Spielrichtung "l" fuer "spiel aufs linke Tor" "r" fuers rechte */
	private char spielrichtung;
  	/** aktueller Spielmodus in String */
 	 private String spielmodus;
 	 /** Checkvariable, ob der Torwart "beamen"(move x y) kann*/
 	 private boolean goalieFree = false;
	/** letzte gehrte Nachricht */
	private String[] hearMsg = {"0", "0", "12", "XyObject"};
    // HearCounter
    private boolean hearnew = false;

  /** empfangenes aktuelles WorldObject */
    private WorldObject aktWO = new WorldObject();
    /** empfangenes aktuelles OverviewObject */
    private OverviewObject aktOvO;

	// Spielerdaten fuer diverse berechnungen...
	/** absolute Position des Spielers */
	protected XyObject mePos;
	/** Ballposition */
	protected XyzObject ballPos;
	/** Kopfdrehung */
	public double meHeadMom;
	/** absolute koerperdrehung */
	public double meBodyMom;
    // Spielstandvariable (array[0] ist eigener, array[1] ist gegner-Spielstand
    private int[] spielstand = {0,0};
	/** Serveradresse */
	private String servAd;
	/** Serverport */
	private int servPort;
	/** Teamname, "Jamaica" */
	private String teamName;
    /** counter fuer moveTo */
    int c = 0;

	private Thread weltbildThread;
	private Thread taktikThread;
  private boolean sleepMode = false;
  int[] arccosArray = new int[2001];
  double[] sin = new double[361];
  double[] cos = new double[361];


	/** Konstruktor mit Parameteruebergabe. Player hat keinen Standartkonstruktor
	 * @author pg
	 * @author fg
	 * @param spielernummerIn Nummer des SPielers (1..11)
	 * @param servAdIn Serveradresse
	 * @param servPortIn Serverport
	 * @param teamName Name des Teams
	 */
	public Player(int spielernummerIn, String servAdIn, int servPortIn, String teamName)
	{
		this.spielernummer = spielernummerIn;
		this.servAd = servAdIn;
		this.servPort = servPortIn;
		this.teamName = teamName;
		/** Erzeugen von Weltbild, Taktik und Parser */
		this.weltb = new Weltbild(this);
		this.takt = new Taktik(this);
		this.pars = new Parser(this.servAd, this.servPort, this);
	  /** Spieler kriegen je nach Nummer ihre Position zugewiesen */
	  switch (spielernummer)
	  {
	    case 1: position='t'; break;
	    case 2: position='a'; break;
	    case 3: position='a'; break;
	    case 4: position='a'; break;
	    case 5: position='m'; break;
	    case 6: position='m'; break;
	    case 7: position='m'; break;
	    case 8: position='m'; break;
	    case 9: position='s'; break;
	    case 10: position='s'; break;
	    case 11: position='s'; break;
	  }
    setArccos();
    for (int i=0; i<=360;i++)
    {
      sin[i] = Math.sin(Math.toRadians(i));
      cos[i] = Math.cos(Math.toRadians(i));
    }

 	}

    /** Wird automatisch gestartet wenn der Thread losgeht. Wenn run() beendet endet auch der Thread
     *@author fg
     */
    public void run()
	{
		getPars().setThread(currentThread());
		getTakt().start();
	    getWeltb().start();
	    if (spielernummer != getPars().getPlayerNum())
	    {
	       System.err.println("Fehler: Vom Server wurde falsche Spielernummer zugewiesen!");
	       System.exit(1);
	    }
		  getAufstellungMitBall(spielernummer);

	  	long i = 0;
        while (true)
        {
	  		sleepMode = false;
	  		getPars().checkNew();
	    	try
	    	{
	    	    sleepMode = true;
	      		currentThread().sleep(5);
	    	}
     		catch (InterruptedException e)
        {
	    	}
    	}
	}

	/** Sendet Init-String an den Server
	 * @author fg
	 */
	public void init()
  {
    String goalie="";
		if (spielernummer == 1) { goalie=" (goalie)"; }
		getPars().transOut("(init "+teamName+" (version 7.0)"+goalie+")");
  }

	// get & set Methoden

	/** gibt die weltbildreferenz zurueck
     * @author pg
     * @return weltb Referenz aufs Weltbild
     */
	public Weltbild getWeltb()
	{
		return this.weltb;
	}

  /** Fllt das arccos-Array mit Werten
   *  @author fg
   */
 	private void setArccos()
  {
    double x;
    for (int i=0; i<=2000; i++)
    {
       x = ((double)i / 1000) - 1;
       arccosArray[i] = (int)Math.round(Math.toDegrees(Math.acos(x)));
    }
  }

  /** Schlgt den Arcus-Cosinus in einer Tabelle nach und liefert den zugehrigen Winkel im Gradmass zurck
   *  @author fg
   *  @param x Double zwischen -1 und 1 von dem der arccos ausgerechnet werdne soll.
   *  @return ArcusCosiuns von x oder 4444 wenn x nicht im bereich von -1 bis 1
   */
  private int arccos(double x)
  {
    if (x < -1 || x > 1) return 4444;
    int pos = (int)((x + 1) * 1000);
    return arccosArray[pos];
  }

	/** gibt die Taktikreferenz zurueck
     * @author pg
     * @return takt Referenz auf Taktik
     */
	public Taktik getTakt()
	{
		return this.takt;
	}

	/** gibt die Parserreferenz zurueck
     * @author pg
     * @return pars Referenz auf den Parser
     */
	public Parser getPars()
	{
		return this.pars;
	}

  public boolean getSleepMode()
	{
	  return sleepMode;
	}
double lastTurn = 0;
boolean lastTurnBoolean = false;

public boolean lastTimeTurn()
{
	return this.lastTurnBoolean;
}
	/** gibt stellt die Spieler auf
	 * @author sg
	 * @int Spnr - Spielernummer
     */
	public void getAufstellungMitBall(int Spnr)
	{
		switch(Spnr)
		{
			//Torwart
			case 1: getPars().transOut("(move -50 0)"); break;
		    //Abwehr
		    case 2: getPars().transOut("(move -28 12)"); break;
		    case 3: getPars().transOut("(move -28 0)"); break;
		    case 4: getPars().transOut("(move -28 -12)"); break;
		    //Mittelfeld
		    case 5: getPars().transOut("(move -16 24)"); break;
		    case 6: getPars().transOut("(move -16 8)"); break;
		    case 7: getPars().transOut("(move -16 -8)"); break;
		    case 8: getPars().transOut("(move -16 -24)"); break;
		    //Sturm
		    case  9: getPars().transOut("(move -1 16)"); break;
			case 10: getPars().transOut("(move -1 0)"); break;
		    case 11: getPars().transOut("(move -1 -16)"); break;
    	}
	}

	public void getAufstellungOhneBall(int Spnr)
	{
		switch(Spnr)
		{
			//Torwart
			case 1: getPars().transOut("(move -50 0)"); break;
		    //Abwehr
		    case 2: getPars().transOut("(move -30 12)"); break;
		    case 3: getPars().transOut("(move -30 0)"); break;
		    case 4: getPars().transOut("(move -30 -12)"); break;
		    //Mittelfeld
		    case 5: getPars().transOut("(move -20 20)"); break;
		    case 6: getPars().transOut("(move -20 8)"); break;
		    case 7: getPars().transOut("(move -20 -8)"); break;
		    case 8: getPars().transOut("(move -20 -20)"); break;
		    //Sturm
		    case  9: getPars().transOut("(move -10 12)"); break;
			case 10: getPars().transOut("(move -10 0)"); break;
		    case 11: getPars().transOut("(move -10 -12)"); break;
	    	}
	}


	/** gibt die Spielernummer zurueck
     * @author pg
     * @return spielernummer Die Nummer des Spielers (1..11)
     */
    public int getSpielernummer()
	{
		return this.spielernummer;
	}

  	private Thread getWeltbildThread()
  	{
  		return this.weltbildThread;
  	}

  	public void setWeltbildThread(Thread weltbildThread)
  	{
  	    this.weltbildThread = weltbildThread;
  	}

  	private Thread getTaktikThread()
  	{
  	    return this.taktikThread;
  	}

  	public void setTaktikThread(Thread taktikThread)
  	{
  	    this.taktikThread = taktikThread;
  	}


	/** gibt die Position des Spielers zurueck
     * @author pg
     * @return position Die Position des Spielers
     * @see Attribut "position"
     */
	public char getPosition()
	{
		return this.position;
	}

	/** gibt die Spielrichtung zurueck
     * @author pg
     * @return spielrichtung Die aktuelle Spielrichtung
     * @see Attribut "spielrichtung"
     */
	public char getSpielrichtung()
	{
		return this.spielrichtung;
	}

    /** gibt den Teamnamen zurueck
     * @author pg
     * @return teamName Der Name des Teams des Spielers
     */
    public String getTeamName()
    {
		return this.teamName;
    }

	/** setzt die Posititon des Spielers auf (pos)
	 * @author pg
	 * @param pos die Position, die der Spieler zugewiesen bekommt
	 * @see Attribut "position"
	 */
	public void setPosition(char pos)
	{
		this.position = pos;
	}

    /** Setzt den Spielmodus, z.B. before_kick_off, play_on, time_over etc
	 * @author pg
	 * @param modus der Spielmodus
	 */
    public void setSpielmodus(String modus)
    {
		this.spielmodus = modus;
		if (modus.startsWith("goalie_catch") && modus.charAt(modus.length()-1)==spielrichtung) goalieFree = true;
	}

	/** Liefert den Spielmodus, z.B. before_kick_off, play_on, time_over etc
	 * @author sg
	 * @Return spielmodus - Der aktuelle Spielmodus
	 */
	public String getSpielmodus()
	{
		return this.spielmodus;
	}

	public boolean getGoalieFree()
	{
	  return goalieFree;
	}

	public void setGoalieFree()
	{
		this.goalieFree = false;
	}

	/** Setzt die Spielrichtung, l oder r fr eigene Seite ist links bzw. rechts
	 * @author pg
	 * @param richtung die Spielrichtung
	 */
	public void setSpielrichtung(char richtung)
	{
		this.spielrichtung = richtung;
	}
	/** Methode zum setzen des Spielstandes nach Abfrage durch z.B. Taktik
	*@param our : Eigener Punktestand
	*@param their : gegnerischer Punktestand
	*@author js
	*/

	public void setSpielstand(int our, int their)
	{
		spielstand[0] = our;
		spielstand[1] = their;
	}
	/** Methode zum erhhen des Spielstandes, falls eine Referee-Message im Parser ankommt!
	*@param side : Spielseite die gepunktet hat
	*@author js
	*/
	public void incrSpielstand(char side)
	{
		if (spielrichtung == side) spielstand[0]++;
		else spielstand[1]++;
	}

	/** Hiermit wird der letzte String, der gesagt wurde gespeichert
	 * @param msg Nachricht
	 */
	public void setLastHear(String[] msg)
	{
	  hearMsg=msg;
      this.hearnew = true;
	}

	/** Liefert die letzte gehrte Meldung zurck
	 * return Letzte Meldung
	 */
	public String[] getLastHear()
	{
	  this.hearnew = false;
      return hearMsg;

	}

    public boolean getHearnew()
    {
        return this.hearnew;
    }

	public ServerParamObj getServerParam()
	{
	  return getPars().getServerParam();
	}

	public PlayerParamObj getPlayerParam()
	{
	  return getPars().getPlayerParam();
	}

	public PlayerTypesObj getPlayerTypes()
	{
	  return getPars().getPlayerTypes();
	}

  public double getBodyMom()
  {
    return this.meBodyMom;
  }

  public void setBodyMom(double winkel)
  {
    this.meBodyMom = winkel;
  }

    // Methoden (Spielerfertigkeiten)

	/** uebergiebt dem Parser den "turn"-Befehl als String mit den Parametern
	 * @author pg
	 * @param moment gibt an um wieviel Grad die Koerperausrichtung gedreht wird
	 */
	public void turn(double moment)
	{

	this.lastTurnBoolean = true;
    this.lastTurn = moment;
    double test = moment;
    double speed = aktOvO.getSpeedAmount();
    /** errechne den Winkel um den bezgl. der momentanen Geschwindigkeit
     *  tatsaechlich gedreht werden muss (nach der Formel im kick Model)*/
    double realturn = moment * (1 + (5 * speed));
    // Mehr wie 180 geht leider nicht
    if (realturn > 180)
    {
      realturn = 180;
      moment = 180 / (1 + (5 * speed));
    }
    // ebenso weniger wie 180
    if (realturn < -180)
    {
      realturn = -180;
      moment = -180 / (1 + (5 * speed));
    }
    // schicke den tatsaechlichen Winkel an den Server
		this.pars.transOut("(turn "+(int)realturn+")");
    // errechne den neuen BodyWinkel (meBodyMom)
		this.meBodyMom = (this.meBodyMom + moment) % 360;
		if (this.meBodyMom <= 0) this.meBodyMom = this.meBodyMom + 360;
        weltb.setBodyAngle(this.meBodyMom);

	}

    /** uebergiebt dem Parser den "turn_neck"-Befehl als String mit den Parametern
	 * @author pg
	 * @param moment gibt an auf wieviel grad die Kopfausrichtung gedreht wird
	 */
	public void turnHead(double moment)
	{
		this.pars.transOut("(turn_neck "+((moment) - (this.meHeadMom))+")");
	}

	/** uebergiebt dem Parser den "turn_neck"-Befehl als String mit den Parametern
	 * @author pg
	 * @param moment gibt an um wieviel grad die Kopfausrichtung gedreht wird
	 */
	public void turnHeadRel(double moment)
	{
		this.pars.transOut("(turn_neck "+(moment)+")");
	}

	 /** Fuehrt Berechnungen durch um sich auf eine bestimmten Position auszurichten. ruft turn auf.
	  * @author pg
	  * @param koord absolute Koordinaten des gewuenschten Ziels
	  * @see turn
	  * @see kick (selbe Winkelberechnung)
	  */
	public void adjust(XyObject koord)
	{
		/** Variable in der die aktuelle Bodyangle gespeichert wird */
		if (this.meBodyMom == 360) this.meBodyMom = 0;
		/** Variable zur zwischenspeicherung des Winkels */
		double turndir = 0;
		/** Berechnung der Ankathete */
		double a = (koord.getX()) - (this.mePos.getX());
		/** Berechnung der Gegenkathete */
		double g = (koord.getY()) - (this.mePos.getY());
		/** Berechnung der Hypothenuse (Pytagoras) */
		double h = Math.sqrt((a * a) + (g * g));
		if (h != 0)
		{
		/** berechnet mit arccos den Winkel im Bogenmass und waehlt die richtige Drehrichtung */
		if (g < 0)
		{
			turndir = arccos(a /h) - this.meBodyMom;
			if (turndir < -180)
			{
				turndir = 360 + turndir;
			}
		}
		else //g >= 0
		{
			turndir = ((360 - arccos(a/h)) - this.meBodyMom);
			if (turndir > 180)
			{
				turndir = turndir - 360;
			}
		}
		/** toleranz fuer den turdir-winkel : 5 grad */
		double absturndir = Math.abs(turndir);
		if (absturndir >= 5)
		    turn(turndir);
		else this.lastTurnBoolean = false;
        }
        // else:(h == 0) du stehst drauf! hoer auf dich zu drehen!
	}

 /** Fuehrt Berechnungen durch um den kopf auf eine bestimmten Position auszurichten. ruft turnHeadRel auf.
	  * @author pg
	  * @param koord absolute Koordinaten des gewuenschten Ziels
	  * @see turnHeadRel
	  * @see turnHead
	  * @see adjust
	  */
	public void adjustHead(XyObject koord)
	{
		/** Variable zur zwischenspeicherung des Winkels */
		double turndir = 0;
		/** Berechnung der Ankathete */
		double a = (koord.getX()) - (this.mePos.getX());
		/** Berechnung der Gegenkathete */
		double g = (koord.getY()) - (this.mePos.getY());
		/** Berechnung der Hypothenuse (Pytagoras) */
		double h = Math.sqrt((a * a) + (g * g));
		if (h != 0)
		{
		    /** berechnet mit arccos den Winkel im Bogenmass und waehlt die richtige Drehrichtung */
		    if (g < 0)
		    {
    			turndir = arccos(a /h) - this.meBodyMom - this.meHeadMom;
	    		if (turndir < -180)
		    	{
			    	turndir = 360 + turndir;
			    }
		    }
		    else //g >= 0
		    {
			    turndir = ((360 - arccos(a/h)) - this.meBodyMom - this.meHeadMom);
			    if (turndir > 180)
			    {
    				turndir = turndir - 360;
	    		}
		    }
            if (turndir + this.meHeadMom < -180 || turndir + this.meHeadMom > 180)
                turnHead(0);
            else // -180 <= turndir + this.meHeadMom <= 180
            {
		        /** toleranz fuer den turndir-winkel : 5 grad */
		        double absturndir = Math.abs(turndir);
		        if (absturndir >= 5)
                    turnHeadRel(turndir);
            }
        }
        // else:(h == 0) du stehst drauf! hoer auf dich zu drehen!
	}



	/** uebergibt dem Parser den "say"-Befehl als String mit den Parametern
	 * @author pg
	 * @param ballkoord Koordinaten des Balls
	 * @param spielernr Spielernummer des angebruellten Spielers
	 * @param ziel Zielkoordinaten fuer den Spieler
	 */
	public void shout(XyzObject ballkoord, int spielernr, XyObject ziel)
	{
		this.pars.transOut("(say ball "+ballkoord.getX()+" "+ballkoord.getY()+" "+ballkoord.getA()+" "+ballkoord.getB()+" "+spielernr+" "+ziel.getX()+" "+ziel.getY()+")");
	}

	/** bewegt den Spieler auf die Koordinaten (koord)
	 * @author pg
	 * @param koord die gewuenschten Koordinaten
	 */
	public void move(XyObject koord)
	{
		this.pars.transOut("(move "+((double)(koord.getX()) / 10)+" "+((double)(koord.getY()) / 10)+")");
	}

    /** berechnet den Winkel um auf eine bestimmte Position zu schiessen, und ruft dann "kickMe" auf
     * @author pg
     * @author sg
     * @param power Staerke des Schusses
     * @param koord Koordinaten des Ziels
     * @see kickMe
     */
	public void kick(XyObject koord, double power)
	{

		/** Variable in der die aktuelle Bodyangle gespeichert wird */
		double bodyangle = this.meBodyMom;
		if (bodyangle == 360) bodyangle = 0;
		/** Variable zur zwischenspeicherung des Winkels */
		double kickdir = 0;
		/** Berechnung der Ankathete */
		double a = (koord.getX()) - (this.mePos.getX());
		/** Berechnung der Gegenkathete */
		double g = (koord.getY()) - (this.mePos.getY());
		/** Berechnung der Hypothenuse (Pytagoras) */
		double h = Math.sqrt((a * a) + (g * g));
		if (h != 0)
		{
		/** berechnet mit arccos den Winkel im Bogenmass und waehlt die richtige Drehrichtung */
		if (g < 0)
		{
			kickdir = arccos(a / h) - bodyangle;
			if (kickdir < -180)
			{
				kickdir = 360 + kickdir;
			}
		}
		else //g >= 0
		{
			kickdir = ((360 - arccos(a / h)) - bodyangle);
			if (kickdir > 180)
			{
				kickdir = kickdir - 360;
			}
		}
		kickMe(power, kickdir);
        }
        // else:(h == 0) du stehst drauf!!
	}

    /** ueberladene kick-methode. berechnet die power um eine bestimmte koordinate zu treffen und ruft dann kcik mit koordinate und power auf
     * @author pg
     * @param koord Koordinaten des Ziels
     * @see kick(XyObject, double)
     */
    public void kick(XyObject koord)
    {
        /** trittkraft fuer den kick mit schussstaerke. initialwert 100 =) */
        double power = 100;
        /** Berechnung der Ankathete */
		double a = (koord.getX()) - (this.mePos.getX());
		/** Berechnung der Gegenkathete */
		double g = (koord.getY()) - (this.mePos.getY());
		/** Berechnung der Hypothenuse (Pytagoras) */
		double h = Math.sqrt((a * a) + (g * g));
        double alpha = (h / 5);
        if (alpha < 100) power = alpha;
        kick(koord, power);
    }

	 /** Fuehrt Berechnungen durch um sich zu einer bestimmten Position zu bewegen. ruft dann turn und dash auf.
	  * @author pg
	  * @author sg (Das hab ich mir verdient *G*)
	  * @param koord absolute Koordinaten des gewuenschten Ziels
	  * @param speed gewuenschte Reisegeschwindigkeit. =)
	  * @see dash
	  * @see turn
	  * @see kick (selbe Winkelberechnung)
	  */
	public void moveTo(XyObject koord, double speed)
	{
		/** Variable in der die aktuelle Bodyangle gespeichert wird */
		if (this.meBodyMom == 360) this.meBodyMom = 0;
		/** Variable zur zwischenspeicherung des Winkels */
		double rundir = 0;

		/** Berechnung der Ankathete */
		double a = (koord.getX()) - (this.mePos.getX());
		/** Berechnung der Gegenkathete */
		double g = (koord.getY()) - (this.mePos.getY());
		/** Berechnung der Hypothenuse (Pytagoras) */
		double h = Math.sqrt((a * a) + (g * g));
		if (h != 0)
        {
		/** berechnet mit arccos den Winkel im Bogenmass und waehlt die richtige Drehrichtung */
		if (g < 0)
		{
			rundir = arccos(a / h) - this.meBodyMom;
			if (rundir < -180)
			{
				rundir = 360 + rundir;
			}
		}
		else //g >= 0
		{
			rundir = ((360 - arccos(a / h)) - this.meBodyMom);
			if (rundir > 180)
			{
				rundir = rundir - 360;
			}
		}
		/** toleranz fuer den rundi-winkel : 5 grad */
		double absrundir = Math.abs(rundir);


		if (absrundir >= 5)
		    turn(rundir);
		else
		{
			this.lastTurnBoolean = false;
		    dash(speed);
        //Falls er nah am Ball steht errechne deine eigene Position und gib sie ans Weltbild weiter
		    if (h < 2*(speed/10))
		    {
          //double speedammount = aktOvO.getSpeedAmount();
          double x = this.mePos.getX() + (this.cos[(int)(360-this.meBodyMom)] * ((speed/10)));
          double y = this.mePos.getY() + (this.sin[(int)(360-this.meBodyMom)] * ((speed/10)));
          weltb.setSelfPos(new XyzObject((int)x,(int)y,4444,4444));
		    }
		 }
        }
        // else:(h == 0)du stehst drauf...
    }

	// interne Methoden (Spielerfertigkeiten die nicht direkt angesteuert werden)

	/** uebergiebt dem Parser den "Kick"-Befehl als String mit den Parametern
	 * @author pg
	 * @param power Trittkraft
	 * @param dir Schussrichtung ("direction")
	 */
    private void kickMe(double power, double dir)
	{
		this.pars.transOut("(kick "+(power)+" "+(dir)+")");
	}

	/** uebergiebt dem Parser den "dash"-Befehl als String mit den Parametern
	 * @author pg
	 * @param speed Laufgeschwindigkeit
	 */
    private void dash(double speed)
	{
		this.pars.transOut("(dash "+(speed)+")");
	}

	// Methoden zur Datenorganisation

	/** leitet das aktuelle WorldObject vom Parser an das Weltbild weiter
     * @author pg
     * @param aktWOIn das aktuelle WorldObject
	 * @see comPack.Parser
	 * @see Weltbild
     */
	public void rcvdWO(WorldObject aktWOIn)
	{
	    this.aktWO = aktWOIn;
	    if (this.weltb.aktualisieren(aktWO))
	    {
	      if (getWeltbildThread() != null)
	      {
	        getWeltbildThread().interrupt();
  	    }
  	  }
 	}

	/** leitet das aktuelle OverviewObject vom Weltbild an Taktik weiter und legt Daten, die fuer Berechnungen benoetigt werden in Variablen ab.
     * @author pg
     * @param ovIn das aktuelle OverviewObject
	 * @see strategie.Taktik
	 * @see Weltbild
     */
	public void incomOV(OverviewObject ovIn)
	{
	  this.aktOvO = ovIn;
		this.mePos = (ovIn.getSelfPos());
		this.ballPos = (ovIn.getBallPos());
		this.meHeadMom = (ovIn.getHeadangle());
		this.meBodyMom = (ovIn.getBodyangle());
		if (this.takt.setOV(ovIn))
		{
		  if (getTaktikThread() != null)
	    {
	      getTaktikThread().interrupt();
  	  }
	  }
  }
    /** Liefert das aktuelle OverviewObjekt auf Anfrage weiter
     * @author fg
     */
    public OverviewObject getOverview()
    {
	return aktOvO;
    }
}
//
/*
 * $Log: Player.java,v $
 * Revision 1.85  2002/07/14 12:07:31  sg
 * Testausgabe entfernt
 *
 * Revision 1.84  2002/07/14 10:33:02  sg
 * Probs mit der Drehung behoben
 *
 * Revision 1.83  2002/07/11 16:56:19  sg
 * System.out.println's eingebaut
 *
 * Revision 1.82  2002/07/10 10:13:05  ms
 * - da muss beim mergen wieder was verschwunden sein
 *
 * Revision 1.81  2002/07/08 07:15:09  fg
 * 5* auf 2* im moveTo gender
 *
 * Revision 1.80  2002/07/08 05:52:58  sg
 * Nur bei getAufstellungOhneBall() die Nr 10 in die Mitte gestellt
 *
 * Revision 1.79  2002/07/08 04:12:36  sg
 * Aufstellung defensiver
 *
 * Revision 1.78  2002/07/08 03:57:58  sg
 * Aufstellung gendert
 *
 * Revision 1.77  2002/07/08 01:14:31  pg
 * adjustHead sieht jetzt ganz gut aus...
 *
 * Revision 1.76  2002/07/08 01:08:45  pg
 * noch nicht ganz fertige adjustHead...
 *
 * Revision 1.75  2002/07/08 01:06:00  ms
 * -- Wenn Y errechnet werden soll, sollte man auch Y Koordinaten nehmen :-)
 *
 * Revision 1.74  2002/07/07 23:59:28  ms
 * - Testausgabe raus
 *
 * Revision 1.73  2002/07/07 23:14:40  ms
 * -- noch eine kleine Aenderung im turn
 *
 * Revision 1.72  2002/07/07 23:00:22  ms
 * -- Turn Methode geaendert
 *     tatsaechlicher Winkel wird jetzt berrechnet
 *
 * Revision 1.71  2002/07/07 22:13:09  sg
 * getGoalieFree() implementiert
 *
 * Revision 1.70  2002/07/07 20:35:20  pg
 * kick mit power noch schwaecher (x/5)
 *
 * Revision 1.69  2002/07/07 18:39:18  pg
 * kick mit Powerberechnung jetzt etwas schwaecher
 *
 * Revision 1.68  2002/07/07 17:43:22  fg
 * *** empty log message ***
 *
 * Revision 1.67  2002/07/07 16:17:45  fg
 * Interpolation der Selfpos nach moveTo
 *
 * Revision 1.66  2002/07/07 14:37:49  fg
 * Fehler mit falschem Blickwinkel rausgemacht, dreht jetzt wieder korrekt
 *
 * Revision 1.65  2002/07/07 03:05:53  sg
 * Aufstellung berarbeitet
 *
 * Revision 1.64  2002/07/06 22:54:25  js
 * Abfrage eingefuegt, so dass Torwart jetzt ueberpruefen kann, ob er den (move x y)-Befehl ausfuehren darf.
 *
 * Revision 1.63  2002/07/06 22:04:18  pg
 * scheiss auf moveto
 *
 * Revision 1.62  2002/07/06 21:58:57  pg
 * moveto2
 *
 * Revision 1.61  2002/07/06 21:41:51  pg
 * neue moveTo probieren...
 *
 * Revision 1.60  2002/07/06 21:24:49  fg
 * Arccos-Methoden kommentiert
 *
 * Revision 1.59  2002/07/06 21:09:27  fg
 * Effiziente Arccos eingebaut
 *
 * Revision 1.58  2002/07/06 20:02:57  sg
 * getAufstellungMitBall() und ohneBall() implementiert
 *
 * Revision 1.57  2002/07/06 20:01:42  sg
 * weitere Auftstellung eingefhrt
 *
 * Revision 1.56  2002/07/06 16:33:12  pg
 * alte moveTo
 *
 * Revision 1.55  2002/07/06 16:25:39  pg
 * neue moveTo
 *
 * Revision 1.54  2002/07/06 15:05:49  pg
 * kcik mit powerberechnung ueberladen
 *
 * Revision 1.53  2002/07/06 14:57:24  pg
 * mensch...schreibfehler in der kommentierung...
 *
 * Revision 1.52  2002/07/06 14:41:34  pg
 * kickMe und dash jetzt wieder private
 *
 * Revision 1.51  2002/07/06 14:31:34  sg
 * Aufstellung auf 3-4-3
 *
 * Revision 1.50  2002/07/05 23:50:44  sg
 * Aufstellung gendert
 *
 * Revision 1.49  2002/07/05 22:23:40  sg
 * Aufstellung berarbeitet
 *
 * Revision 1.48  2002/07/05 19:53:13  pg
 * adjust implementiert
 *
 * Revision 1.47  2002/07/05 19:22:47  sg
 * Kleinigkeit
 *
 * Revision 1.46  2002/07/05 17:14:46  sg
 * neuste Version
 *
 * Revision 1.45  2002/07/04 13:40:56  ms
 * - das '' in den Positionen beseitigt
 *
 * Revision 1.44  2002/07/03 11:47:26  pg
 * kick und moveTo geaendert ... =)
 *
 * Revision 1.43  2002/07/03 06:48:58  sg
 * Kleinigkeit
 *
 * Revision 1.42  2002/07/02 21:51:56  fg
 * Fehler (hoffentlich) ausgemerzt, der dafr sorgen konnte das jeder Thread evtl. abgeschossen werden kann.
 *
 * Revision 1.41  2002/07/02 17:58:19  sg
 * Aufstellung leicht gendert..
 *
 * Revision 1.40  2002/06/28 14:11:37  sg
 * Hab leider ka mehr was ich alles gendert hab ;)
 *
 * Revision 1.39  2002/06/27 18:52:59  sg
 * Die Methode getLasthear verursachte Fehler - wurden jetzt behoben :)
 *
 * Revision 1.38  2002/06/27 18:21:22  sg
 * Methode run() leicht verndert, auerdem eine neue Methode (getAufstellung()) eingefhrt.
 *
 * Revision 1.37  2002/06/27 07:32:14  sg
 * Die Methode getSpielmodus eingefgt..
 *
 * Revision 1.36  2002/06/26 18:52:56  fg
 * - hearMsg ist String[] da mehr als ein Wort drinstehen kann
 *
 * Revision 1.35  2002/06/26 11:09:58  pg
 * fehlertoleranz auf 2 grad geaendert
 *
 * Revision 1.34  2002/06/26 10:02:02  pg
 * wo war meine if-schleife???
 *
 * Revision 1.33  2002/06/26 09:57:58  pg
 * kleine aenderung in der berechnung fuer moveto, kick, etc...
 *
 * Revision 1.32  2002/06/26 09:43:02  pg
 * ups, kleiner fehler gefunden...
 *
 * Revision 1.31  2002/06/26 09:37:19  pg
 * thread sleep wieder rausgenommen, muss irgendwie wieder reingerutscht sein
 *
 * Revision 1.30  2002/06/26 09:29:50  pg
 * - ein paar attribute und die referenzen auf protected gesetzt, wegen des torwarts...
 *
 * Revision 1.29  2002/06/25 21:00:59  fg
 * turn befehl in der moveTo() Methode auskommentiert, berechnung ist falsch, hat den Spieler immer falsch gedreht.
 *
 * Revision 1.28  2002/06/25 20:34:50  fg
 * *** empty log message ***
 *
 * Revision 1.27  2002/06/25 16:32:52  fg
 * getOverview eingbaut
 *
 * Revision 1.26  2002/06/21 16:45:06  sg
 * Ein paar Plusis eingefgt damit man wieder kompilieren kann
 *
 * Revision 1.25  2002/06/20 10:54:40  pg
 * - shout jetzt mit neuen Parametern (xyz, int, xy)
 *
 * Revision 1.24  2002/06/20 10:17:16  sg
 * Wie besprochen "dribbeln" entfernt
 *
 * Revision 1.23  2002/06/19 14:49:30  js
 * kleine Bugfixes, damit's wieder compiliert ;-)
 *
 * Revision 1.22  2002/06/19 14:30:58  fg
 * - get/setLastHear eingebaut
 * - getServerParam, getPlayerParam und getPlayerTypes eingebaut
 *
 * Revision 1.21  2002/06/19 10:35:34  pg
 * - neuer moveTo
 *
 * Revision 1.20  2002/06/19 10:07:38  js
 * Korrigierte Version nach dem 0-Byte-Fehler
 *
 * Revision 1.17  2002/06/18 22:31:54  js
 * *** empty log message ***
 *
 * Revision 1.16  2002/06/18 14:08:53  fg
 * Auf 1.14 wiedereingespielt, die 1.15 war kaputt
 *
 * Revision 1.14  2002/06/17 19:50:18  fg
 * - Taktik Thread wird gestartet
 * - get/setTaktikThread und get/setWeltbildThread eingefgt
 * - Thread.interrupt() wird an WeltbildThread bzw. TaktikThread geschickt, wenn ein Objekt abgeliefert wurde
 *
 * Revision 1.13  2002/06/17 14:50:28  fg
 * yield in der run() wurde durch Thread.sleep(10) ersetzt,
 * mit entsprechender Exception-Behandlung, und
 * interrupt "von unten".
 * Player.java: Weltbild Thread wird gestartet
 *
 * Revision 1.12  2002/06/17 10:43:29  pg
 * - import java.lang.math fuer winkelberechnung
 * - meHeadMom, meBodyMom, mePos fuer berechnungen
 * - incomOV implementiert
 * - turnHead, move, moveTo, kick, dribble und abbruch implementiert
 *
 * Revision 1.7  2002/06/12 10:12:25  ms
 * - getTeamName implementiert
 *
 * Revision 1.6  2002/06/10 12:41:17  pg
 * - torschusss methode raus
 * -kick heisst jetzt kickMe (interne methode)
 * -pass heisst jetzt kick (weil mehr als nur pass moeglich)
 *
 * Revision 1.5  2002/06/10 12:31:01  js
 * Fehler in der Kommentierung beseitigt(jetzt wieder compilierbar)
 *
 * Revision 1.4  2002/06/10 12:11:03  pg
 * -dash, kick, turn, shout, turnHeadRel implementiert
 * -getWO durch rcvdWO ersetzt
 * -turnHead-Body hinzugefuegt
 * -move und moveTo mit XyObjects...
 * -JavaDoc Kommentiert
 *
 * Revision 1.3  2002/06/07 22:15:23  fg
 * - Constructor angepasst
 * - Aufruf des Parsers angepasst
 * - run() Methode eingefgt
 * - init() Methode zum Anmelden am Server implementiert
 *
 * Revision 1.2  2002/06/07 19:15:53  fg
 * Aufruf von Parser an neues Format angepasst
 *
 * Revision 1.1.1.1  2002/06/05 10:38:57  pg
 * intital version
 *
 *
 */

