/*
 * $Id: SeeObject.java,v 1.15 2002/07/08 04:46:41 pg Exp $
 *
 */

/**
 * Objektklasse.
 * speichert visuelle daten vom server
 * @author Jens Spiess (js)
 * @author Patrick Gaertner (pg)
 *
 */

public class SeeObject
	extends WorldObject
{
	// Attribute
    /** array mit 1..11 Spielern mit distance, direction, dist.change, dir.change, bodyfacingdir, headfacingdir, [evtl goalie] */
	private double[][] players = new double[11][7];
    /** array mit 1..11 Spielern mit distance, direction, dist.change, dir.change, bodyfacingdir, headfacingdir, [evtl goalie] */
	private double[][] enemys = new double[11][7];
    /** array mit max. 21 unbekannten Spielern mit (s.o.) und team, falls zugehoerigkeit bekannt, aber nicht Spielernummer */
	private double[][] unknowns;
    /** array mit den festen koordinaten der unbeweglichen Objekte */
	private double[][] flags = new double[55][4];
    /** koordinaten des balls */
	private double[] ball = new double[4];
	/** gesehene Linie mit Winkel und Entfernung: Welche Linie man sieht ist line[2]: 1=rechte Linie;2=untere Linie; 3=linke Linie; 4=obere Linie*/
	private double[] line = new double[3]; 
	/** der aktuelle tick */
	private int timestep ;
	
    // Konstruktor
    /** konstruktor. setzt isC, die players und die flags
     * @author pg
     * @author js
     */
	public SeeObject()
	{
	    this.setIsC(true);
	    for (int i=0; i<11;i++)
	    for (int z=0 ; z<7; z++)
	    players[i][z] = 4444;
	    for (int k=0; k<55; k++)
        for (int z=0; z<4; z++)
        flags[k][z] = 4444;
	}
	
	// get & set methoden
	/** gibt den Playerarray zurueck
	 * @author pg
	 * @return players der Playerarray
	 */
	public double[][] getPlayers()
	{
		return this.players;
	}
	
    /** gibt den Timestep zurueck
     * @author js
     * @return timestep der tick
     */ 
	public int getTimestep()
	{
	  return timestep;
	}
	
	/** gibt den Enemyarray zurueck
	 * @author pg
	 * @return enemys der Enemyarray
	 */
	public double[][] getEnemys()
	{
		return this.enemys;
	}
	
    /** gibt den Unbekannte Spielerarray zurueck
     * @author pg
     * @return unknowns der Array der unbekannten Spieler
     */
	public double[][] getUnknowns()
	{
		return this.unknowns;
	}
	
    /** gibt das Array der Flags zurueck
     * @author pg
     * @return flags der Flagarray
     */
	public double[][] getFlags()
	{
		return this.flags;
	}
    
    /** gibt die Linie zurueck
     * @author js
     * @return line die Linie
     */
	public double[] getLine()
	{
	  return line;
	}
	
    /** gibt den Ball zurueck
     * @author pg
     * @return ball
     */
	public double[] getBall()
	{
		return this.ball;
	}
	
    /** setzt den tick
     * @author js
     * @param step der Wert der Timestep zugewiesen wird
     */
	public void setTimestep(int step)
	{
	  timestep=step;
	}
	
	
    /** setzt die Spielerwerte (0..5) des Spielers (pos) der eigenen Mannschaft
	 * @author pg
	 * @param pos Spielernummer
	 * @param dist Entfernung
	 * @param dir Winkel
	 * @param distChng Entfernungsaenderung
	 * @param dirChng Winkelaenderung 
	 * @param bodfcngdir Koerperausrichtung
	 * @param headfcngdir Kopfdrehungswinkel
	 * @see RCSManual
     */
	public void setPlayers(int pos, double dist, double dir, double distChng, double dirChng, double bodfcngdir, double headfcngdir, double goalie)
	{
		this.players[pos-1][0] = dist;
		this.players[pos-1][1] = dir;
		this.players[pos-1][2] = distChng;
		this.players[pos-1][3] = dirChng;
		this.players[pos-1][4] = bodfcngdir;
		this.players[pos-1][5] = headfcngdir;
		this.players[pos-1][6] = goalie;
	}
	
    /** setzt die Spielerwerte (0..5) des Spielers (pos) der Feindmannschaft
	 * @author pg
	 * @param pos Spielernummer
	 * @param dist Entfernung
	 * @param dir Winkel
	 * @param distChng Entfernungsaenderung
	 * @param dirChng Winkelaenderung
	 * @param bodfcngdir Koerperausrichtung
	 * @param headfcngdir Kopfdrehungswinkel
	 * @see RCSManual
     */
  public void setEnemys(int pos, double dist, double dir, double distChng, double dirChng, double bodfcngdir, double headfcngdir, double goalie)
	{
		this.enemys[pos-1][0] = dist;
		this.enemys[pos-1][1] = dir;
		this.enemys[pos-1][2] = distChng;
		this.enemys[pos-1][3] = dirChng;
		this.enemys[pos-1][4] = bodfcngdir;
		this.enemys[pos-1][5] = headfcngdir;
		this.enemys[pos-1][6] = goalie;
	}
	
    /** setzt die Spielerwerte (0..5) eines unbekannten Spielers
	 * @author pg
	 * @param speicherort
	 * @param dist Entfernung
	 * @param dir Winkel
	 * @param distChng Entfernungsaenderung
	 * @param dirChng Winkelaenderung
	 * @param bodfcngdir Koerperausrichtung
	 * @param headfcngdir Kopfdrehungswinkel
	 * @param team teamzugehoerigkeit
	 * @see RCSManual
     */
	public void setUnknowns(double dist, double dir, double distChng, double dirChng, double bodfcngdir, double headfcngdir, double team)
	{
	  
	  if (unknowns == null) unknowns = new double[1][7];
	  else unknownAdd(unknowns);
	  int aktIndex = unknowns.length - 1;

	  this.unknowns[aktIndex][0] = dist;
		this.unknowns[aktIndex][1] = dir;
		this.unknowns[aktIndex][2] = distChng;
		this.unknowns[aktIndex][3] = dirChng;
		this.unknowns[aktIndex][4] = bodfcngdir;
		this.unknowns[aktIndex][5] = headfcngdir;
		this.unknowns[aktIndex][6] = team;
		
	}
    
    /** fuegt dem Array der unbekannten Spieler einen weiteren solchen hinzu
     * @author js
     * @param inArr der Ausgangsarray
     */
	private void unknownAdd(double[][] inArr)
	{
	  int laenge = inArr.length;
	  double[][] unK = new double[laenge+1][7];
	  System.arraycopy(inArr,0,unK,0,inArr.length);
	  unknowns = unK;
	}

    /** setzt den Flagarray
     * @author js
     * @param numb nummer der flag
     * @param distance entfernung zur flag
     * @param direction winkel zur flag
     * @param distCH distancechange zur flag
     * @param dirCH directionchange
     */
	public void setFlags(int numb, double distance, double direction, double distCH, double dirCH)
	{
		this.flags[numb][0] = distance;
		this.flags[numb][1] = direction;
  	    this.flags[numb][2] = distCH;
		this.flags[numb][3] = dirCH;
	}

    /** setzt den Wert der Linie
     * @author js
     * @param dist entfernung zur Linie
     * @param dir Winkel zur Linie
     * @param lindex nummer der Linie 
     */
    public void setLine(double dist, double dir, double lIndex)
	{
	  line[0] = dist;
	  line[1] = dir;
	  line[2] = lIndex;
	}

    /** setzt den Ball
     * @author js
     * @param dist Entfernung zum Ball
     * @param dir Winkel zum Ball
     * @param distCh distancechange zum Ball
     * @param dirCh directionchange zum Ball
     */
	public void setBall(double dist, double dir, double distCh, double dirCh)
	{
		this.ball[0] = dist;
		this.ball[1] = dir;
		this.ball[2] = distCh;
		this.ball[3] = dirCh;
	}
}

/*
 * $Log: SeeObject.java,v $
 * Revision 1.15  2002/07/08 04:46:41  pg
 * JavaDoc Kommentiert =) final Version
 *
 * Revision 1.14  2002/07/06 23:00:17  js
 * timestep-Variable eingefuegt
 *
 * Revision 1.13  2002/07/05 18:56:51  js
 * kleines BugFixelchen, mit ArrayOutOfBounds
 *
 * Revision 1.12  2002/07/05 18:40:23  js
 * Flags werden jetzt auch direkt mit 4444 initialisiert
 *
 * Revision 1.11  2002/07/04 12:44:55  js
 * Das setzen der Unbekannten Spieler ist jetzt dynamisch...Das entspr. Array wird also autom. verlngert
 *
 * Revision 1.10  2002/06/26 11:56:15  fg
 * initialisieren von player
 *
 * Revision 1.9  2002/06/26 09:51:39  pg
 * doc
 *
 * Revision 1.8  2002/06/20 13:57:48  js
 * setLine, getLine und line zugefgt!
 * Methoden und Variable zum rechnen mit der gesehenen Linie!
 *
 * Revision 1.7  2002/06/17 04:34:47  js
 * Krorrektur der bergabe ausm Parser(bergabeArray-1)
 *
 * Revision 1.6  2002/06/14 20:44:51  js
 * Die Methoden setPlayers und setEnemys wurden um die Information (goalie) erweitert;
 * die Methode setUnknowns muss insoweit berarbeitet werden, das sie ein Array dynamischer Laenge anlegt!(naeheres dazu im Kommentar vor selbiger Methode)
 *
 * Revision 1.5  2002/06/11 16:51:17  js
 * Habe den Ball wie es sich gehrt mit 4 Eigenschaften ausgestattet...:Dist, Dir, DistCh, DirCh
 *
 * Revision 1.4  2002/06/11 15:13:59  js
 * Wenn der Parser(Ver. 1.11) setFlags aufruft, gibt es einen "ArrayIndexOutOfBounce"
 *
 * Revision 1.3  2002/06/10 22:27:01  js
 * einige Kleinigkeiten korrigiert (Syntaktisch)
 *
 * Revision 1.2  2002/06/10 13:20:19  pg
 * - player, enemys und unknowns array geaendert nach jens-specifications
 * - kommentiert
 * -setPlayer, -enemys, -unknowns an neue def. angepasst
 * -SETBALL DIR UND DIST VERTAUSCHT!!!
 *
 * Revision 1.1.1.1  2002/06/05 10:38:57  pg
 * intital version
 *
 *
 */

