/*
 * $Id: Kommunikation.java,v 1.11 2002/07/08 06:01:37 js Exp $
 *
 */

/**
 * Serverkommunikation.
 * sendet und empfaengt.
 * 
 * @author Florian Grassl (fg)
 * @author Jens Spies (js)
 *
 */

package comPack;

import java.io.*;
import java.net.*;

class Kommunikation extends Thread
{
  /** Hier wird der Servername gespeichert */
  private String serverName; 
  /** Hier wird der Serverport gespeichert */
  private int serverPort;
  /** Dies ist die Referenz auf den Parser, der das Kommunikationsobjekt erzeugt hat */
  private Parser parser;
  /** Socket, wird zum versenden und empfangen bentigt */
  private DatagramSocket socket;
  /** ServerAdresse in ein Format umgewandelt das die Klassen aus java.net.* verstehen */
  private InetAddress serverAddress;
  /** Verweis auf den Playerthread */
  private Thread playerThread;
  
  /** Konstructor mit Uebergabeparametern. Kommunikation hat keinen Standartkonstruktor.
   *  @param serverName Der beim Programmstart bergebene ServerName
   *  @param serverPort Der beim Programmstart bergebene ServerPort
   *  @param parser Referenz auf das bergeordnete Parser-Objekt
   */
  Kommunikation(String serverName, int serverPort, Parser parser)
  {
	this.serverName=serverName;
    this.serverPort=serverPort;
	this.parser=parser;
 	try 
	 	{
      /** construct datagram socket and bind it to any available port on the local machine */
      this.socket        = new DatagramSocket();
      /** set target address */
      this.serverAddress = InetAddress.getByName(serverName);
      /** set target port */
      this.serverPort    = serverPort;
    } 
    catch (SocketException se) 
    {
      System.err.println();
      System.err.println("Fehler: Der vom Server zugewiesene Port ist bereits an ein anderes Programm vergeben!");
      System.exit(1);
    } 
    catch (UnknownHostException uhe) 
    {
      System.err.println("Der eingegebene Server konnte nicht gefunden werden!");
      System.err.println("Bitte folgende Richtlinien:");
      System.err.println();
      usage();
      System.exit(1);
    }
  }
  private static void usage()
  {
    System.out.println("Programmaufruf:");
    System.out.println();
    System.out.println("java Jamaica Teamname Servername ServerPort [String]");
    System.out.println();
    System.out.println("Das Programm muss mit keinem, 3 oder optional 4 Parametern gestartet werden!");
    System.out.println("Ohne Paramter werden Defaultwerte verwendet!");
    System.out.println();
    System.out.println("Teamname  : Ist der Name der Mannschaft im Monitor (default: Jamaica)");
    System.out.println("Servername: IP-Adresse oder DNS-Name des Servers (default: localhost)");
    System.out.println("Serverport: Port des Robo-Servers.Positiver Ganzzahlwert (default: 6000)");
    System.out.println("[String]  : Ein Beliebiger String beliebiger Laenge.");
    System.out.println("             Veranlasst den Start der Visualisierung.");
    System.out.println();
  }
  
  /** setzt den Playerthread
   * @author fg
   * @author js
   * @param playerThread der Verweis auf den Playerthread
   */
  void setThread(Thread playerThread)
  {
    this.playerThread=playerThread;
  }

  /** gibt den Verweis auf den Playerthread zurueck
   * @author fg
   * @author js
   * @return playerThread der Verweis auf den Playerthread
   */
  private Thread getPlayerThread()
  {
    return this.playerThread;
  }
  

  /** Wird automatisch ausgefhrt wenn der Thread mittels Kommunikation.start() gestartet wird
   *  und ruft die Methode receive() in einer Endlosschleife auf
   * @author fg
   * @author js
   */
  public void run()
  {
	  while (true)
		{
		  receive();  
		}
  }
  

  /** Methode versendet ber den eingestellten Port eine Nachricht an den Server
   *  @param message Textstring der in dieser Form an den Server bermittelt wird
   */
  void send(String message) 
  {
  	try 
   	{
   	  /** convert the message to a byte array */
   	  byte[] buffer = message.getBytes();
   	  /** create the packet from the buffer and set the server's IP address and port */
   	  DatagramPacket packet = new DatagramPacket(buffer, buffer.length, serverAddress, serverPort);
      /** send the packet */
      socket.send(packet);
	  } 
  	catch (IOException ioe) 
    {
   	  System.err.println("CommunicationChannel: Could not send datagram packet.");
    }
  }

  /** Diese Methode wird automatisch ausgefhrt, wenn der Thread gestartet wird.
   *  Sie wartet solange bis sie vom Server etwas empfngt, und wenn etwas kam, wird
   *  dieser String an den bergeordneten Parser mittels der Methode <code>Parser.received(String)</code>
   *  bergeben
   * @author fg
   * @author js
   */
  void receive() 
  {
    /** allocate buffer for server message */
    byte[] buffer = new byte[4096];
    /** create datagram packet for receiving */
    DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
    try 
    {
      /** receive packet, the method blocks until a datagram arrives */
      socket.receive(packet);
    } 
    catch (IOException ioe) 
    {
      System.err.println("CommmunicationChannel: Could not receive datagram packet.");
    }
    /** get the server port for the following actions */
    serverPort = packet.getPort();
    /** bermitteln des empfangenen Strings an Parser.received */
    if (parser.received(new String(packet.getData()).substring(0, packet.getLength()-1)+" "))
    {
      if (getPlayerThread() != null) 
	    { 
	      getPlayerThread().interrupt(); 
  	  }
    }
  }
}

/*
 * $Log: Kommunikation.java,v $
 * Revision 1.11  2002/07/08 06:01:37  js
 * Fehlerbehandlung mit entsprechenden Ausgaben eingefuegt
 *
 * Revision 1.10  2002/07/08 03:59:10  pg
 * JavaDoc kommentiert =)
 *
 * Revision 1.9  2002/07/02 21:51:56  fg
 * Fehler (hoffentlich) ausgemerzt, der dafr sorgen konnte das jeder Thread evtl. abgeschossen werden kann.
 *
 * Revision 1.8  2002/06/27 11:41:51  fg
 * Receive Puffer von 1024 auf 4096 bytes angehoben
 *
 * Revision 1.7  2002/06/17 19:48:02  fg
 * Thread.interrrupt() wird jetzt an Player-Thread geschickt, wenn ein neuer String empfangen wurde
 *
 * Revision 1.6  2002/06/17 14:50:28  fg
 * yield in der run() wurde durch Thread.sleep(10) ersetzt,
 * mit entsprechender Exception-Behandlung, und
 * interrupt "von unten".
 * Player.java: Weltbild Thread wird gestartet
 *
 * Revision 1.5  2002/06/09 13:37:41  fg
 * javadoc Dokumentationen eingefgt
 *
 * Revision 1.4  2002/06/07 19:12:28  fg
 * *** empty log message ***
 *
 * Revision 1.3  2002/06/06 11:35:46  fg
 * Package wieder eingefgt
 *
 * Revision 1.2  2002/06/05 14:17:37  fg
 * Ersetzen durch "unsere" Version
 *
 * Revision 1.1.1.1  2002/06/05 10:38:57  pg
 * intital version
 *
 *
 */

