/*
 * $Id: Parser.java,v 1.47 2002/07/08 05:17:07 js Exp $
 *
 */

/**
  * "Uebersetzer"
  * Konvertiert Nachrichten vom Server und erstellt das WorldObject.
  * Konvertiert Nachrichten fuer den Server.
  * @author Florian Grassl (fg)
  * @author Jens Spies (js)
  */

package comPack;

import SeeObject;
import SenseObject;
import WorldObject;
import ServerParamObj;
import PlayerParamObj;
import PlayerTypesObj;
import Player;

public class Parser
{
  
  // Referenzen
  /** Referenz auf Kommunikation */
  private Kommunikation communicate;
  /** Referenz auf den Player */
  private Player player;
  
  // Attribute
  /** das aktuelle WorldObject */
  private WorldObject aktWO; 
  /** Object in der die uebermittelten ServerParameter abgelegt werden */
  private ServerParamObj serverParam;       
  /** Object in dem die vom Server uebermittelten PlayerParameter abgelegt werden */
  private PlayerParamObj playerParam;       
  /** Object in dem die Playertypen abgelegt werden */
  private PlayerTypesObj playerTypes;       
  /** temporre Variablen, solange "der Player" noch mit parsing beschftigt ist */
  private String rcvdSeeString = null;     
   /** temporre Variablen, solange "der Player" noch mit parsing beschftigt ist */
  private String rcvdSenseString = null;    
   /** temporre Variablen, solange "der Player" noch mit parsing beschftigt ist */
  private String rcvdHearString = null;     
   /** temporre Variablen, solange "der Player" noch mit parsing beschftigt ist */
  private String rcvdScoreString = null;                
  /** uebergebene Parameter vom Player zur Anmeldung und Kommunikation mit dem Server */
  private String serverName;                
  /** uebergebene Parameter vom Player zur Anmeldung und Kommunikation mit dem Server */
  private int serverPort;                                    
  /** Ist die vom Server zugewiesene Spielernummer, die im Player berprft wird */
  private int playerNum;
  /** Spielrichtung l oder r, l fuer wir spielen auf linker Seite usw. */
  private char playDir;
  /** Wird auf Antwort vom Server gesetzt und dem Player uebergeben */
  private String playMode;                  
  /** Verweis auf den Playerthread */
  private Thread playerThread;              
  /** Variable die auf true gesetzt wird, wenn eine Linie geparst wurde, ansonsten bekommt sie einen Initalwert */
  private boolean parsedLine = false;
  /**  Variable die auf true gesetzt wird, wenn der Ball geparst wurde, ansonsten bekommt er den Initalwert 4444 */
  private boolean parsedBall = false;      
  
  // Konstruktor
  /** Konstruktor mit Parameteruebergabe. Parser hat keinen Standartkonstruktor.
   * @author fg
   * @author js
   * @param serverName Name des Servers, zB. localhost
   * @param serverPort Portnummer zB. 6000 fuer localhost
   * @param player Playerreferenz
   */
  public Parser(String serverName, int serverPort, Player player)
  { 
    this.serverName=serverName;
    this.serverPort=serverPort;
    this.player=player;
    /** Hier wird die Kommunikation mit dem Server erzeugt */
    communicate=new Kommunikation(serverName, serverPort, this);    
    /** startet die Kommunikation */
    communicate.start();                                            
  }

  // interne Methoden

  /** Methode zum weiterleiten von Nachrichten vom Player an den Server
    * @author fg
    * @author js
    * @param outmeg Die Nachricht die vom Player ankommt
    */
  public void transOut(String outmeg)
  {
    communicate.send(outmeg);
  } 


  /** Methode wird aus der Player.run() regelmssig gestartet und berprft, ob ein
    * String zum Parsen vom Kommunikations-Objekt geliefert wurde, und startet dann
    * die entsprechenden parse-Methoden.
    * @author fg
    * @author js
    */
   public void checkNew()
   {
     /** == null wre keine neue See-Nachricht */
     if (rcvdSeeString != null)       
     {
       /** Temporrer speicher */
       String temp = rcvdSeeString;
       /** Klassen-variable wieder auf null setzen */
       rcvdSeeString = null;          
       /** parseSee aufrufen mit aktuellem See-String. */
       parseSee(temp);                 
     }
     /** SenseString wird analog zu SenseString (oben) bearbeitet */
     if (rcvdSenseString != null)
     {
       String temp = rcvdSenseString;
       rcvdSenseString = null;
       parseSense(temp);
     }
     /** HearString wird analog zu SenseString (oben) bearbeitet */
     if (rcvdHearString != null)    
     {
       String temp = rcvdHearString;
       rcvdHearString = null;
       parseHear(temp);
     }
     /** ScoreString wird analog zu SenseString (oben) bearbeitet */
     if (rcvdScoreString != null)   
     {
       String temp = rcvdScoreString;
       rcvdScoreString = null;
       parseScore(temp);
     }
   }
    
   /** setzt den Threadverweis
    * @author fg
    * @author js
    * @param playerThread der Thread
    */
   public void setThread(Thread playerThread)
   {
     communicate.setThread(playerThread);
   }


  /** bekommt vom receiveThread die empfangenen Nachrichten bergeben, 
    * berprft ob die empfangene Nachricht vom Typ see, hear, sense oder score sind.
    * einmalige Strings (ini,server_param,player_param), und errors(die nicht auftreten sollten) werden sofort geparst
    * @author fg
    * @author js
    * @param rcvdString bergebener String aus dem receiveThread
    */ 
  boolean received(String rcvdString)
  {
    if (rcvdString.startsWith("(see")) { this.rcvdSeeString = rcvdString; if (playerThread != null) playerThread.interrupt(); }
    if (rcvdString.startsWith("(sense")){ this.rcvdSenseString = rcvdString; if (playerThread != null)  playerThread.interrupt(); }
    if (rcvdString.startsWith("(hear")) { this.rcvdHearString = rcvdString; if (playerThread != null)  playerThread.interrupt(); }
    if (rcvdString.startsWith("(score")) { this.rcvdScoreString = rcvdString; if (playerThread != null)  playerThread.interrupt(); }

    /** Folgende Methoden werden sofort geparsed, da sie entweder nur zu Beginn des Spiels kommen oder nicht autreten sollten ;-) */
    if (rcvdString.startsWith("(error")) parseError(rcvdString);
    if (rcvdString.startsWith("(ini")) parseInit(rcvdString);
    if (rcvdString.startsWith("(server_param")) parseServerParam(rcvdString);
    if (rcvdString.startsWith("(player_param")) parsePlayerParam(rcvdString);
    if (rcvdString.startsWith("(player_type")) parsePlayerType(rcvdString);
    /** liefert zurck, ob der Player gerade "schlaeft"(true), oder ob er gerade schon etwas parsed */
    return player.getSleepMode();           
  }

   /** Wertet einen eingegangenen init-String aus und ber gibt die Parameter 
    * spielrichtung und spielmodus per set-Methoden an den player
    * @author fg
    * @param rcvdString eingegangene Servernachricht
    */
  private void parseInit(String rcvdString)
  {
    /** Aufstckelung des String */
    String[] strArray = createArray(rcvdString.substring(1,rcvdString.length()-2)," ");
    /** Setzen der klassenvariablen */
    this.playDir= strArray[1].charAt(0);              
    this.playerNum= Integer.parseInt(strArray[2]);    
    this.playMode= strArray[3];                       
    /** bergabe an den Player */
    player.setSpielrichtung(this.playDir);
    player.setSpielmodus(this.playMode);  
  }
  
  /** parsed die eintreffenden see-Nachrichten vom Server, und wandelt sie in ein SeeObject um.
    * Dies wird dann an den Player bergeben
    * @author js
    * @param rcvdString eingegangene Servernachricht
    */
  private void parseSee(String rcvdString)
  {
    /** Der Ball wird bei eingehender Nachricht als "nicht gesehen" markiert */
    parsedBall=false;        
    /** Linie wird bei eingehender Nachricht als "nicht gesehen" markiert */
    parsedLine=false;        
    /** neues SeeObject anlegen das spter uebergeben wird */
    SeeObject aktSeeObj = new SeeObject();    
    String[][][] strArray = createArray(rcvdString.substring(1,rcvdString.length()-2),"(",")"," ");     //konvertierung des Strings zum verarbeiten
    aktSeeObj.setTimestep(Integer.parseInt(strArray[0][0][1]));
    for (int i=1 ; i < strArray.length ; i++)
    {
      /** Hier beginnt die Flagzuweisung */
      if (strArray[i][0][0].charAt(0)=='f')
      {
        if ((strArray[i][0].length) == 2)
        {
          switch (strArray[i][1].length)
          {
            case 4: aktSeeObj.setFlags(27,Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),Double.parseDouble(strArray[i][1][2]),Double.parseDouble(strArray[i][1][3]));break;
            case 2: aktSeeObj.setFlags(27,Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),4444,4444); break;
            case 1: aktSeeObj.setFlags(27,Double.parseDouble(strArray[i][1][0]),4444,4444,4444); break;
          }
        }             
        if (strArray[i][0].length == 3)
        {
          int k = flagNumber(3,strArray[i][0][0] + strArray[i][0][1] + strArray[i][0][2]);
          switch (strArray[i][1].length)
          {
            case 4: aktSeeObj.setFlags(k,Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),Double.parseDouble(strArray[i][1][2]),Double.parseDouble(strArray[i][1][3]));break;
            case 2: aktSeeObj.setFlags(k,Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),4444,4444); break;
            case 1: aktSeeObj.setFlags(k,Double.parseDouble(strArray[i][1][0]),4444,4444,4444); break;
          }
        }    
        if (strArray[i][0].length == 4)
        {
          int k = flagNumber(4,strArray[i][0][0] + strArray[i][0][1] + strArray[i][0][2] + strArray[i][0][3]);
          switch (strArray[i][1].length)
          {
            case 4: aktSeeObj.setFlags(k,Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),Double.parseDouble(strArray[i][1][2]),Double.parseDouble(strArray[i][1][3]));break;
            case 2: aktSeeObj.setFlags(k,Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),4444,4444); break;
            case 1: aktSeeObj.setFlags(k,Double.parseDouble(strArray[i][1][0]),4444,4444,4444); break;
          }
        }
      }
      /** Hier endet die Flagzuweisung */
          
      /** Dies ist die Abfrage der Tore */
      if (strArray[i][0][0].equals("g") || strArray[i][0][0].equals("G"))
      {
        if (strArray[i][0].length==2 && strArray[i][0][1].equals("l"))
        {
          switch (strArray[i][1].length)
          {
            case 4: aktSeeObj.setFlags(21,Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),Double.parseDouble(strArray[i][1][2]),Double.parseDouble(strArray[i][1][3]));break;
            case 2: aktSeeObj.setFlags(21,Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),4444,4444);break;
            case 1: aktSeeObj.setFlags(21,4444,Double.parseDouble(strArray[i][1][0]),4444,4444);break;
          }
        }
        if (strArray[i][0].length==2 && strArray[i][0][1].equals("r"))
        {
          switch (strArray[i][1].length)
          {
            case 4: aktSeeObj.setFlags(33,Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),Double.parseDouble(strArray[i][1][2]),Double.parseDouble(strArray[i][1][3]));break;
            case 2: aktSeeObj.setFlags(33,Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),4444,4444);break;
            case 1: aktSeeObj.setFlags(33,4444,Double.parseDouble(strArray[i][1][0]),4444,4444);break;
          }
        }
        else
        {
          /** Falls Tor hinter sich gesehen wird kann es keine Zuweisung stattfinden... */
        }
      }
      /** Ende der Abfrage der Tore */
          
      /** Zuweisung des Balles */
      if (strArray[i][0][0].equals("b") || strArray[i][0][0].equals("B"))
      {
        parsedBall=true;
        int fuell = 4444;
        if (strArray[i][0][0].equals("B")) fuell = -3333;
        switch (strArray[i][1].length)
        {
          case 4: aktSeeObj.setBall(Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),Double.parseDouble(strArray[i][1][2]),Double.parseDouble(strArray[i][1][3]));break;
          case 2: aktSeeObj.setBall(Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),fuell,fuell);break;
          case 1: aktSeeObj.setBall(fuell,Double.parseDouble(strArray[i][1][0]),fuell,fuell);break;
        }
      }
      /** Ende der Ballzuweisung */
      
      /** Anfang der Zuweisungen gesehener Spieler */
      if (strArray[i][0][0].equals("p"))
      {
        if (strArray[i][0].length == 1)
        {
          if (strArray[i].length > 1)
          {
            int Infos = strArray[i][1].length;
            setUnKnowns(aktSeeObj,Infos,strArray[i][1],2,false);
          }
        }
        if (strArray[i][0].length == 2)
        {
          if (strArray[i].length > 1)
          {
            int Infos = strArray[i][1].length;
            if (strArray[i][0][1].substring(1,strArray[i][0][1].length()-1).equals(player.getTeamName())) setUnKnowns(aktSeeObj,Infos,strArray[i][1],1,false);
            else setUnKnowns(aktSeeObj,Infos,strArray[i][1],0,false);
          }
        }
        if (strArray[i][0].length == 3)
        {
          if (strArray[i].length > 1)
          {
            int Unum = Integer.parseInt(strArray[i][0][2]);
            int Infos = strArray[i][1].length;
            if (strArray[i][0][1].substring(1,strArray[i][0][1].length()-1).equals(player.getTeamName())) setPLayers(aktSeeObj,Unum,Infos,strArray[i][1],0);
            else setENemys(aktSeeObj,Unum,Infos,strArray[i][1],0);
          }
        }
        if (strArray[i][0].length == 4)  
        {
          if (strArray[i].length > 1)
          {
            int Unum = Integer.parseInt(strArray[i][0][2]);
            int Infos = strArray[i][1].length;
            if (strArray[i][0][1].substring(1,strArray[i][0][1].length()-1).equals(player.getTeamName())) setPLayers(aktSeeObj,Unum,Infos,strArray[i][1],1);
            else setENemys(aktSeeObj,Unum,Infos,strArray[i][1],1);
          }
        }
      }
      /** Ende der Spielerzuweisungen */
      
      /** Zuweisugen der Objekte in der Visible-Distance */
      if (strArray[i][0][0].charAt(0)=='P')
      {
        if (strArray[i].length > 1)
        {
          int Infos = strArray[i][1].length;
          setUnKnowns(aktSeeObj,Infos,strArray[i][1],2,true);
        }
      }
      /** Ende der zuweisung Visible_distance */
      
      /** Zuweisung der gesehenen Linie */
      if (strArray[i][0][0].charAt(0)=='l')
      {
        parsedLine=true;
        double lIndex =0;
        if (strArray[i][0].length>1)
        {
          switch (strArray[i][0][1].charAt(0))
          {
            case 'r': lIndex = 1; break;
            case 'l': lIndex = 3; break;
            case 't': lIndex = 4; break;
            case 'b': lIndex = 2; break;
          }
        }
        if (strArray[i].length>1)
        {
          if (strArray[i][1].length==2) aktSeeObj.setLine(Double.parseDouble(strArray[i][1][0]),Double.parseDouble(strArray[i][1][1]),lIndex);
          else aktSeeObj.setLine(4444,Double.parseDouble(strArray[i][1][0]),lIndex);
        }
      }
      /** Ende Zuweisung der Linien */
    }
    /** Falls der Ball nicht gesehen wurde, wird die Position usw. auf 4444 gesetzt */
    if (parsedBall==false) aktSeeObj.setBall(4444,4444,4444,4444); 
    /** Falls keine Linie gesehen wurde, wird die Position und Winkel auf 4444 gesetzt und LinienNummer auf 0 */
    if (parsedLine==false) aktSeeObj.setLine(4444,4444,0); 
    /** hier wird das gefuellte SeeObject zum Player weitergegeben */
    player.rcvdWO(aktSeeObj); 
  }
  /** Ende des See-Parsing */

  /** konvertiert/speichert die empfangenen eigenen Spieler in das aktSeeObj
    * @author js
    * @param seeO von parseSee empfangenes seeObject
    * @param PlNr Spielernummer die gesetzt wird
    * @param infos Anzahl der Informationen die man ber einen Spieler hat
    * @param stringArr entsprechendes Array mit den Informationen ueber den Spieler
    * @param goalie ist der gesehene Spieler ein Torwart?
    * @see parseSee
    */
  private void setPLayers(SeeObject seeO,int PlNr, int infos, String[] stringArray, double goalie)
  {
    switch (infos)
    {
      case 6: seeO.setPlayers(PlNr, Double.parseDouble(stringArray[0]), Double.parseDouble(stringArray[1]), Double.parseDouble(stringArray[2]), Double.parseDouble(stringArray[3]), Double.parseDouble(stringArray[4]), Double.parseDouble(stringArray[5]),goalie);break;
      case 2: seeO.setPlayers(PlNr, Double.parseDouble(stringArray[0]), Double.parseDouble(stringArray[1]), 4444,4444,4444,4444,goalie);break;
      case 1: seeO.setPlayers(PlNr, 4444,Double.parseDouble(stringArray[0]),4444,4444,4444,4444,goalie);break;
    }
  }
  
  /** konvertiert/speichert den empfangenen Gegenspieler in das aktSeeObj
    * author js
    * @param seeO von parseSee empfangenes seeObject
    * @param PlNr Spielernummer die gesetzt wird
    * @param infos Anzahl der Informationen die man ber einen Spieler hat
    * @param stringArr entsprechendes Array mit den Informationen ueber die Gegner
    * @param goalie ist der gesehene Spieler ein Torwart?
    * @see parseSee
    */
  private void setENemys(SeeObject seeO,int PlNr, int infos, String[] stringArray, double goalie)
  {
    switch (infos)
    {
      case 6: seeO.setEnemys(PlNr, Double.parseDouble(stringArray[0]), Double.parseDouble(stringArray[1]), Double.parseDouble(stringArray[2]), Double.parseDouble(stringArray[3]), Double.parseDouble(stringArray[4]), Double.parseDouble(stringArray[5]),goalie);break;
      case 2: seeO.setEnemys(PlNr, Double.parseDouble(stringArray[0]), Double.parseDouble(stringArray[1]), 4444,4444,4444,4444,goalie);break;
      case 1: seeO.setEnemys(PlNr, 4444,Double.parseDouble(stringArray[0]),4444,4444,4444,4444,goalie);break;
    }
  } 
  
  /** konvertiert/speichert den empfangenen unbekannten Spieler in das aktSeeObj
    * author js
    * @param seeO von parseSee empfangenes SeeObject
    * @param Infos anzahl der Informationen, die man ber den Spieler hat
    * @parma stringArray Array mit den Informationen ber den Spieler
    * @param team Teamzugehrigkeit (o=Gegner,1=eigener,2=unbekannt)
    * @param hinten Gibt an ob der Spieler nur zu weit weg ist(false), oder ob er in der vis_distance (true) ist
    * @see parseSee
    */ 
  private void setUnKnowns(SeeObject seeO,int Infos, String[] stringArray,double team,boolean hinten)
  {
    int fuell = 4444;
    if (hinten) fuell= -3333;
    switch (Infos)
    {
      case 6: seeO.setUnknowns(Double.parseDouble(stringArray[0]), Double.parseDouble(stringArray[1]), Double.parseDouble(stringArray[2]), Double.parseDouble(stringArray[3]), Double.parseDouble(stringArray[4]), Double.parseDouble(stringArray[5]),team);break;
      case 2: seeO.setUnknowns(Double.parseDouble(stringArray[0]), Double.parseDouble(stringArray[1]), fuell,fuell,fuell,fuell,team); break ;
      case 1: seeO.setUnknowns(fuell,Double.parseDouble(stringArray[0]),fuell,fuell,fuell,fuell,team);break;
    }
  }
  
  
  /** errechnet den Index der gesehene Flag
    * author js
    * @param possible Anzahl der Flagbuchstaben
    * @param parsedFlag zusammengefgter String des Flag-Arrays
    * @return flagNum der index der gesehenen Flag 
    */
  private int flagNumber(int possible,String parsedFlag)
  {
    /** Auswahl aus 55 Flags... */
    int flagNum = -2;
    if (possible == 3)
    {
      if (parsedFlag.charAt(2) == 't')
      {
        switch (parsedFlag.charAt(1))
        {
          case 'l' : flagNum = 23; break;
          case 'c' : flagNum = 36; break;
          case 'r' : flagNum = 35; break;
        }
      }
      if (parsedFlag.charAt(2) == 'b')
      {
        switch (parsedFlag.charAt(1))
        {
          case 'l' : flagNum = 19; break;
          case 'c' : flagNum = 18; break;
          case 'r' : flagNum = 31; break;
        }
      }
      if (parsedFlag.charAt(2) == '0')
      {
        switch (parsedFlag.charAt(1))
        {
          case 'l' : flagNum = 9; break;
          case 'r' : flagNum = 45; break;
          case 't' : flagNum = 54; break;
          case 'b' : flagNum = 0; break;
        }
      }
    }
    if (possible == 4)
    {
      if (parsedFlag.substring(1,3).equals("bl"))
      {
        switch (parsedFlag.charAt(3))
        {
          case '1' : flagNum = 1; break;
          case '2' : flagNum = 2; break;
          case '3' : flagNum = 3; break;
          case '4' : flagNum = 4; break;
          case '5' : flagNum = 5; break;
        }
      }
      if (parsedFlag.substring(1,3).equals("br"))
      {
        switch (parsedFlag.charAt(3))
        {
          case '1' : flagNum = 37; break;
          case '2' : flagNum = 38; break;
          case '3' : flagNum = 39; break;
          case '4' : flagNum = 40; break;
          case '5' : flagNum = 41; break;
        }
      }
      if (parsedFlag.substring(1,3).equals("tl"))
      {
        switch (parsedFlag.charAt(3))
        {
          case '1' : flagNum = 17; break;
          case '2' : flagNum = 16; break;
          case '3' : flagNum = 15; break;
          case '4' : flagNum = 14; break;
          case '5' : flagNum = 13; break;
        }
      }
      if (parsedFlag.substring(1,3).equals("tr"))
      {
        switch (parsedFlag.charAt(3))
        {
          case '1' : flagNum = 53; break;
          case '2' : flagNum = 52; break;
          case '3' : flagNum = 51; break;
          case '4' : flagNum = 50; break;
          case '5' : flagNum = 49; break;
        }
      }
      if (parsedFlag.substring(1,3).equals("lt"))
      {
        switch (parsedFlag.charAt(3))
        {
          case '1' : flagNum = 10; break;
          case '2' : flagNum = 11; break;
          case '3' : flagNum = 12; break;
        }
      }
      if (parsedFlag.substring(1,3).equals("lb"))
      {
        switch (parsedFlag.charAt(3))
        {
          case '1' : flagNum = 8; break;
          case '2' : flagNum = 7; break;
          case '3' : flagNum = 6; break;
        }
      }
      if (parsedFlag.substring(1,3).equals("rt"))
      {
        switch (parsedFlag.charAt(3))
        {
          case '1' : flagNum = 46; break;
          case '2' : flagNum = 47; break;
          case '3' : flagNum = 48; break;
        }
      }
      if (parsedFlag.substring(1,3).equals("rb"))
      {
        switch (parsedFlag.charAt(3))
        {
          case '1' : flagNum = 44; break;
          case '2' : flagNum = 43; break;
          case '3' : flagNum = 42; break;
        }
      }
      if (parsedFlag.substring(1,3).equals("pl"))
      {
        switch (parsedFlag.charAt(3))
        {
          case 't' : flagNum = 26; break;
          case 'c' : flagNum = 25; break;
          case 'b' : flagNum = 24; break;
        }
      }
      if (parsedFlag.substring(1,3).equals("pr"))
      {
        switch (parsedFlag.charAt(3))
        {
          case 't' : flagNum = 30; break;
          case 'c' : flagNum = 29; break;
          case 'b' : flagNum = 28; break;
        }
      }
      if (parsedFlag.substring(1,3).equals("gl"))
      {
        switch (parsedFlag.charAt(3))
        {
          case 't' : flagNum = 22; break;
          case 'b' : flagNum = 20; break;
        }
      }
      if (parsedFlag.substring(1,3).equals("gr"))
      {
        switch (parsedFlag.charAt(3))
        {
          case 't' : flagNum = 34; break;
          case 'b' : flagNum = 32; break;
        }
      }
    }
    return(flagNum);
  }    
  
  /** Parsed die Hear-Messages
    * Goal messages werden direkt verarbeitet
    * die Playmodes ber den Referee werden direkt in den Player geschrieben
    * Spielernachrichten werden auch direkt in den Player geschrieben (lastHear)
    * @author js
    * @param rcvdString die eingegangene Nachricht
    */
  private void parseHear(String rcvdString)
  { 
    String[] strArray  = createArray(rcvdString.substring(1,rcvdString.length()-2)," ");
    String[] msg;
    if (strArray[2].equals("referee")) parseRef(strArray[3]);
    else
    {
      msg = new String[strArray.length-3];
      System.arraycopy(strArray, 3, msg, 0, strArray.length-3);
      msg[0] = msg[0].substring(1);
      if (msg.length > 0) msg[msg.length-1] = msg[msg.length-1].substring(0, msg[msg.length-1].length()-1);
      player.setLastHear(msg);
    } 
  }
  
  /** uebergibt eingegangene Referee-Messages an den Player
    * @author js
    * @param plMode = der bergebene Playmode
    */
  private void parseRef(String plMode)
  {
    if (plMode.startsWith("goal_"))
    {
      switch (plMode.charAt(5))
      {
        case 'k': player.setSpielmodus(plMode); break;
        default :  player.incrSpielstand(plMode.charAt(5)); player.setSpielmodus(plMode.substring(0,6)); break;
      }
    }
    else player.setSpielmodus(plMode);
  }
  
  /** Diese Methode Uebergibt dem Player die einkommende sense_body Nachricht als WorldObject(SenseObject)
    * @author js
    * @param rcvdString einkommende sense-Body-Nachricht
    */
  private void parseSense(String rcvdString)
  {
    SenseObject aktSenObj = new SenseObject();
    String[][] strArray = createArray(rcvdString.substring(1,rcvdString.length()-2),"("," ");
    aktSenObj.setTimestep(Integer.parseInt(strArray[0][1]));
    for (int z=1; z<strArray.length;z++)
    {
      int help=strArray[z].length-1;
      strArray[z][help]=strArray[z][help].substring(0,strArray[z][help].length()-1);
      switch (z)
      {
        // auskommentierter Quellcode. im zuge der wiederverwendbarkeit nicht entfernt. =)
        //case 1: aktSenObj.setView(strArray[z][2],strArray[z][3]);         
        case 2: aktSenObj.setStamina(Double.parseDouble(strArray[z][1]));/* aktSenObj.setEffort(Integer.parseInt(strArray[z][2]));*/ break;
        case 3: aktSenObj.setSpeedamount(Double.parseDouble(strArray[z][1]));aktSenObj.setSpeeddirection(Double.parseDouble(strArray[z][2]));break;
        case 4: aktSenObj.setHeadangle(Double.parseDouble(strArray[z][1]));break;
        //case 5: player.setKickCount(Integer.parseInt(strArray[z][1]));break;
        //case 6: player.setDashCount(Integer.parseInt(strArray[z][1]));break;
        //case 7: player.setTurnCount(Integer.parseInt(strArray[z][1]));break;
        //case 8: player.setSayCount(Integer.parseInt(strArray[z][1]));break;
        //case 9: player.setTurnNeckCount(Integer.parseInt(strArray[z][1]));break;
        //case 10: player.setCatchCount(Integer.parseInt(strArray[z][1]));break;
        //case 11: player.setMoveCount(Integer.parseInt(strArray[z][1]));break;
        //case 12: player.setChangeViewCount(Integer.parseInt(strArray[z][1]));break;
        default : break;
      }
    }
    player.rcvdWO(aktSenObj);
  }
  
  /** parsed die einkommende (score)-Nachrichten (, falls diese vom Programm aus abgefragt wird)
    * @author js
    * @param rcvdString eingegangene Score-Nachricht
    */
  private void parseScore(String rcvdString)
  {
    String[] stand = createArray(rcvdString.substring(1,rcvdString.length()-2)," ");
    player.setSpielstand(Integer.parseInt(stand[2]),Integer.parseInt(stand[3]));
  }

  
  /** ruft die vom Server zugewiesene Spielernummer ab
    * @author fg    
    * @return playerNum Nummer des Spielers
    */
  public int getPlayerNum()
  {
    return this.playerNum;
  }
  
  /** Gibt aus falls Fehlermeldungen vom Server kommen(, dies solte aber nicht vorkommen)
    * @author js
    * @param rcvdString eingegangener Error-String
    */
  private void parseError(String rcvdString)
  {
    if (rcvdString.startsWith("(error no_more_"))
    { 
      System.out.println("Das gewaehlte Team besteht bereits, und hat 11 Spieler auf dem Platz!");
      System.exit(1);
    }
    
    System.out.println("Spieler " + player.getSpielernummer() + " bekam folgenden Fehler gemeldet: " + rcvdString);
  }
  
  /** uebergibt die eingegangene aufgeschluesselte Server_Param-Nachricht
    * @author fg
    * @param rcvdString eingegangener Serverparameter-String
    */
  private void parseServerParam(String rcvdString)
  {
    serverParam = new ServerParamObj(createArray(rcvdString.substring(1,rcvdString.length()-2), " "));
  }
  
  /** uebergibt die eingegangene aufgeschluesselte Player_Param-Nachricht
    * @author fg
    * @param rcvdString eingegangener Playerparameter-String
    */
  private void parsePlayerParam(String rcvdString)
  {
    playerParam = new PlayerParamObj(createArray(rcvdString.substring(1,rcvdString.length()-2), " "));
  }
  
  /** uebergibt die eingegangene aufgeschluesselte Player_Type-Nachricht
    * @author fg
    * @param rcvdString eingegangener Playerparameter-String
    */
  private void parsePlayerType(String rcvdString)
  {
    playerTypes = new PlayerTypesObj(createArray(rcvdString.substring(1,rcvdString.length()-2), " "));
  }
  
  /** gibt das Server_Param-Object zurueck
    * @author fg
    * @return serverParam die Serverparameter
    */
  public ServerParamObj getServerParam()
  {
    return serverParam;
  }
  
  /** gibt das Player_Param-Object zurueck
    * @author fg
    * @return playerParam die Playerparameter
    */
  public PlayerParamObj getPlayerParam()
  {
    return playerParam;
  }
  
  /** gibt das Player_Types-Object zurueck
    * @author fg
    * @return playerTypes die Playerttypen
    */
  public PlayerTypesObj getPlayerTypes()
  {
    return playerTypes;
  }
  
  /** Methode zum aufschluesseln eingegangener Servernachrichten
    * @author fg
    * @param origString String der aufgesplittet werden soll
    * @param trennzeichen Zeichen nachdem gesplittet werden soll
    * @return strArray mit Array, nach einem Trennzeichen sortiert
    */
  private String[] createArray(String origString, String trennzeichen)
  {
    String[] strArray=new String[0];
    int index=origString.indexOf(trennzeichen);
    while (index != -1)
    {
      /** = Erstes Zeichen ist nicht das Trennzeichen */
      if (index != 0) 
      {
        strArray=StrArrayAdd(strArray,origString.substring(0,index));
      }
      origString=origString.substring(index+1);
      index=origString.indexOf(trennzeichen);
    }
    /** 0 = Letztes Zeichen war auch Trennzeichen */
    if (origString.length() != 0) 
      strArray=StrArrayAdd(strArray,origString);
    return strArray;
  }
 
  /** Methode zum aufschluesseln eingegangener Servernachrichten
    * @author fg
    * @param origString String der aufgesplittet werden soll
    * @param trennzeichen1 Zeichen nachdem gesplittet werden soll
    * @param trennzeichen2 Zeichen nachdem gesplittet werden soll
    * @return testA2 Array, nach zwei Trennzeichen sortiert
    */
  private String[][] createArray(String origString, String trennzeichen1, String trennzeichen2)
  {
    String[] testA=createArray(origString.trim(), trennzeichen1);
    String[][] testA2=new String[testA.length][];
    for (int i=0; i<testA2.length; i++)
    {
      testA2[i]=createArray(testA[i].trim(),trennzeichen2);
    }
    return testA2;
  }
  
  /** Methode zum aufschluesseln eingegangener Servernachrichten
    * @author fg
    * @param origString String der aufgesplittet werden soll
    * @param trennzeichen1 Zeichen nachdem gesplittet werden soll
    * @param trennzeichen2 Zeichen nachdem gesplittet werden soll
    * @param trennzeichen3 Zeichen nachdem gesplittet werden soll
    * @return testA3 Array, nach drei Trennzeichen sortiert
    */
  private String[][][] createArray(String origString, String trennzeichen1, String trennzeichen2, String trennzeichen3)
  {
    String[][] testA2 = createArray(origString.trim(), trennzeichen1, trennzeichen2);
    String[][][] testA3=new String[testA2.length][][];
    for (int i=0; i<testA2.length; i++)
    {
      testA3[i] = new String[testA2[i].length][];
    }
    for (int i=0; i<testA3.length; i++)
    {
      for (int z=0; z<testA3[i].length; z++)
      {
        testA3[i][z]=createArray(testA2[i][z].trim(),trennzeichen3);
      }
    }
    return testA3;
  }

  /** Methode um das Array dynamisch in der Lnge anzupassen
    * @author fg
    * @param str bisheriger String
    * @param add String der angefuegt werden soll
    * @return s neuer String mit bisherigem String und neuem String hinten angehaengt
    */
  private String[] StrArrayAdd(String[] str, String add)
  {
    String[] s=new String[str.length+1];
    System.arraycopy(str, 0, s, 0, str.length);
    s[s.length-1]=add;
    return s;
  }
}

/*
 * $Log: Parser.java,v $
 * Revision 1.47  2002/07/08 05:17:07  js
 * Die Fehlermeldung "error no_more_team_or_players_or_goalie" wird jetzt mit einem System.exit() behandelt
 *
 * Revision 1.46  2002/07/08 03:50:00  pg
 * JvaDoc Kommentiert =)
 *
 * Revision 1.45  2002/07/07 16:13:40  fg
 * Timestamp im Senseobject setzen
 *
 * Revision 1.44  2002/07/06 23:05:49  js
 * Im aktuellen SeeObject wird jetzt der Timestep gesetzt
 *
 * Revision 1.43  2002/07/06 19:10:14  js
 * Tabs durch Leerzeichen ersetzt
 *
 * Revision 1.42  2002/07/06 18:25:31  js
 * hoffentlich korrektes JavaDoc eingefuegt...
 * sieht man halt erst wenn alles ge-javaDoct ist...
 *
 * Revision 1.41  2002/07/06 13:51:02  js
 * SenseObject
 *
 * Revision 1.40  2002/07/06 13:35:11  js
 * Auskommentierung der Weiterlweitung der Sense-Objects entfernt *oops* :-)
 *
 * Revision 1.39  2002/07/04 19:24:21  js
 * Ich geh zwar nicht davon aus das wir vom Server Fehler bekommen, hab aber mal fr den Fall der Flle ne Textausgabe reingemacht...
 *
 * Revision 1.38  2002/07/02 21:51:56  fg
 * Fehler (hoffentlich) ausgemerzt, der dafr sorgen konnte das jeder Thread evtl. abgeschossen werden kann.
 *
 * Revision 1.37  2002/06/27 10:15:37  js
 * wieder kleinigkeit bei den PlayModes
 *
 * Revision 1.36  2002/06/27 09:12:55  js
 * kleinigkeit beim Play-Mode parsing
 *
 * Revision 1.35  2002/06/26 20:17:35  fg
 * Kleiner fehler im Hear-Parsing behoben
 *
 * Revision 1.34  2002/06/26 18:53:41  fg
 * Hear-message parsing angepasst an situation das Message aus mehr als einem Wort besteht.
 * Schickt jetzt ein String-Array
 *
 * Revision 1.33  2002/06/26 11:57:50  fg
 * Testausgaben auskommentiert
 * P-Parsing abgesichert
 *
 * Revision 1.32  2002/06/26 08:02:22  js
 * Kleine Bugfixes bzw. Ausnahmebehandlung
 *
 * Revision 1.31  2002/06/25 16:31:19  fg
 * Stamina ist double nicht int, wurde geaendert
 *
 * Revision 1.30  2002/06/24 11:45:33  js
 * BugFix:
 * Wenn die Spieler den Ball gar nicht gesehen haben, wurde der Absand zum Ball als 0 uebergeben;
 * Wurde jetzt wie geplant auf 4444 gesetzt (gleiches gilt fr Winkel,...)
 *
 * Revision 1.29  2002/06/22 08:59:43  js
 * Uebergabewerte gesetzt, falls der Spieler keine Linie sieht!
 *
 * Revision 1.28  2002/06/20 13:58:14  js
 * Parsen der gesehenen Linien hinzugefuegt
 *
 * Revision 1.27  2002/06/19 14:49:30  js
 * kleine Bugfixes, damit's wieder compiliert ;-)
 *
 * Revision 1.26  2002/06/19 14:31:37  fg
 * getServerParam, getPlayerParam und getPlayerTypes eingebaut
 *
 * Revision 1.25  2002/06/19 12:31:22  js
 * Da iss schon der erste:
 * die Hear-Msg uebergabe richtiggestellt
 *
 * Revision 1.24  2002/06/19 12:29:09  js
 * Parsing sollte jetzt komplett sein!!
 * evtl. noch Schoenheitsfehler, aber das wars!
 *
 * Revision 1.23  2002/06/18 21:06:44  js
 * Parsing ist soweit komplett fertig...
 * Sense-Nachrichten werden noch nicht bergeben, da im Weltbild entspr. Abfrage fehlt..
 * Hear Nachrichten werden bis auf Spielernachrichten auch geparst...Fr Spielernachrichten muessen noch Absprachen getroffen werden..
 *
 * Revision 1.22  2002/06/18 11:38:55  js
 * Angefangen Hear-Messages zu parsen - Angefangen mit referee_messages
 *
 * Revision 1.21  2002/06/17 19:48:23  fg
 * setThread leitet den PlayerThread einfach an das Kommunikationsobjekt weiter
 *
 * Revision 1.20  2002/06/17 14:50:28  fg
 * yield in der run() wurde durch Thread.sleep(10) ersetzt,
 * mit entsprechender Exception-Behandlung, und
 * interrupt "von unten".
 * Player.java: Weltbild Thread wird gestartet
 *
 * Revision 1.19  2002/06/15 21:39:57  js
 * kleinen Bug beseitigt
 *
 * Revision 1.18  2002/06/15 15:24:37  js
 * Ojekte die hinter einem Spieler sind werden jetzt auch geparst (in die Unknowns)
 *
 * Revision 1.17  2002/06/15 00:16:38  js
 * Kleine Bugfixes am Playerparsing
 *
 * Revision 1.16  2002/06/14 20:41:13  js
 * Status: Es werden jetzt alle Flags(inkl. Tore), der Ball und alle sorten von Spielern(Eigene,Gegner und Unbekannte) mit Angabe ber Torwart geparst
 *
 * Revision 1.15  2002/06/12 14:41:46  fg
 * In-Game parsing wurde vom Kommunikations-Thread abgekoppelt und luft nun im Player-Thread.
 * Kommunikation liefert nur noch beim Parser ab, und empfngt dann wieder.
 * Player ruft regelmssig checkNew() auf um zu berprfen ob was neues kam
 *
 * Revision 1.14  2002/06/12 12:11:10  js
 * Playerparsing  angefangen....wird "zuhause" fortgesetzt
 *
 * Revision 1.13  2002/06/11 16:50:27  js
 * Parsed jetz Flags, Tore und den Ball!
 *
 * Revision 1.12  2002/06/11 16:00:17  js
 * See-Nachrichten werden jetzt in Bezug auf Flags und die Tore richtig bergeben!
 *
 * Revision 1.11  2002/06/11 15:10:18  js
 * Ist mit dem Projekt komplett compilierbar, aber whrend der laufzeit wird mit SeeObj.setFlags ein Array-"Out of Bounce"-Fehler erzeugt
 *
 * Revision 1.10  2002/06/10 22:28:04  js
 * See-Nachrichten stehen kurz vorm parsen...(3 Zeilen sind noch falsch, aber der Algorithmus mte stimmen)
 *
 * Revision 1.9  2002/06/10 17:17:07  fg
 * *** empty log message ***
 *
 * Revision 1.8  2002/06/10 17:09:19  js
 * Verbesserte CreateArray-Methode
 *
 * Revision 1.7  2002/06/10 15:11:22  js
 * parseInit() hat jetzt seine volle Funktion und bergibt die Parameter an den player
 *
 * Revision 1.6  2002/06/09 12:54:39  fg
 * createArray methode verbessert
 *
 * Revision 1.5  2002/06/07 22:17:40  fg
 * Referenz auf Player in Constructor eingebaut
 *
 * Revision 1.4  2002/06/07 19:14:11  fg
 * Init-Meldungen werden jetzt korrekt geparsed, und in entsprechenden Objekten gespeichert.
 *
 * Revision 1.3  2002/06/06 11:35:46  fg
 * Package wieder eingefgt
 *
 * Revision 1.2  2002/06/05 14:17:37  fg
 * Ersetzen durch "unsere" Version
 *
 * Revision 1.1.1.1  2002/06/05 10:38:57  pg
 * intital version
 *
 *
 */

